;;;;============================================================================
;;;;               Modalys Library
;;;;
;;;; authors: Nicholas Ellis
;;;; date: 2003
;;;; © IRCAM 2003
;;;;============================================================================

(in-package cl-user)
(defvar *produce-samples* 1)
(export '*produce-samples*)

(defpackage "MODALYS" 
  (:nicknames "MLYS"))

(in-package "MODALYS")
(defvar *modalys-in-OM* t)


;--------------------------------------------------
;Variable definiton with files to load 
;--------------------------------------------------

(defvar *mos-directory* nil)

;(setq ol (om::lib-pathname om::*current-lib*))
(setq ol (make-pathname :directory (pathname-directory *load-pathname*)))


(let* ((root 
#+mcl       (second (pathname-directory  (ccl::boot-directory)))
#+lispworks  "/"
                      )
       (default  (make-pathname :directory (list :absolute root "Applications" "Modalys")))
       (modalys-alias nil))
  (if (and default (probe-file default))
    (setq *mos-directory* default)

    (let ((modalys-alias-file  (make-pathname :directory (pathname-directory ol) :name "modalys" :type "txt"))
          (modalys-alias nil))
      (if (probe-file modalys-alias-file)
        (setq modalys-alias (pathname (with-open-file (s modalys-alias-file :direction :input)
                                        (read-line s)))))
      (if (or (not modalys-alias-file)
              (not (and modalys-alias (probe-file modalys-alias)))
              (not (probe-file (concatenate 'string (directory-namestring  modalys-alias) "Components"))))
        (progn
          (loop

            while (or (not modalys-alias) 
                      (not (probe-file  (concatenate 'string (directory-namestring  modalys-alias) "Components"))))
            do
            (setq modalys-alias
                  (progn
                    (let ((message "Where is the 'Modalys' folder ?"))
                      (print message)
#+mcl                      (choose-directory-dialog :window-title message)
#+lispworks                (capi:prompt-for-directory message)
                  ))))

          (with-open-file (s modalys-alias-file :direction :output :if-exists :supersede)
            (format s (namestring modalys-alias))))
        (setq modalys-alias (with-open-file (s modalys-alias-file :direction :input)
                              (read-line s))))
      (setq *mos-directory* modalys-alias))))


(setq *mos-directory* (merge-pathnames (make-pathname :directory '(:relative "Components")) *mos-directory*))



(defvar *mlys-init-dir* nil)
(setq *mlys-init-dir* (merge-pathnames (make-pathname :directory '(:relative "init" "lisp")) *mos-directory*))

(defvar *modalys-lib-files* nil)
(setq  sources (merge-pathnames (make-pathname :directory '(:relative "sources")) ol))

(defun corrected-name (x)
  (concatenate 'string x (if (< om::*om-version* 5.01) "_500")))

(setf *modalys-lib-files* (list
                           (merge-pathnames (make-pathname :name "init" :type "lisp") *mlys-init-dir*)
                           (merge-pathnames (make-pathname :name (corrected-name "documentation") :type "lisp") *mlys-init-dir*)
                           (merge-pathnames (make-pathname :name "bpf-bundle" :type "lisp") sources)
                           (merge-pathnames (make-pathname :name "chords-utils" :type "lisp") sources)
                           (merge-pathnames (make-pathname :name "om-modalys" :type "lisp") sources)
                           (merge-pathnames (make-pathname :name "modal-data" :type "lisp") sources)))


;--------------------------------------------------
;Loading files 
;--------------------------------------------------

#+mcl (load (merge-pathnames (make-pathname :name (corrected-name "special") :type "cfsl") *mlys-init-dir*))

(mapc #'load *modalys-lib-files*)


;--------------------------------------------------
; OM subpackages initialization
; ("sub-pack-name" subpacke-lists class-list function-list class-alias-list)
;--------------------------------------------------

(with-package-iterator (ns  (find-package "MODALYS") :internal) (loop
       (multiple-value-bind (more? symbol) (ns)
         (if more? 
            (export symbol "MODALYS")
            (return)))))



(defvar *subpackages-list* nil)
(setf *subpackages-list* 
      '(
        ("General" nil nil (
                            mlys:new 
                            mlys:run 
                            mlys:play 
                            mlys:synthesize
                            ) nil)
        
        ("Objects" nil (
                        bi-string
                        bi-two-mass
                        circ-membrane
                        cello-bridge
                        clamped-circ-plate
                        clone
                        closed-closed-tube
                        closed-open-tube
                        finite-element
                        free-circ-plate
                        harmonic-oscillator
                        jet
                        melt-hybrid
                        mix-hybrid
                        mono-string
                        mono-two-mass
												multiple-points
                        open-open-tube
                        read-from-file
                        rect-free-bar
                        rect-membrane
                        rect-plate
                        reson-model
                        single-point
                        tri-hybrid
                        violin-bridge
                        ) nil)
        ("Controllers" nil (
                            access-force
                            access-position
                            access-speed
                            arithmetic
                            bandlimited-noise
                            constant
                            constant-second-order-filter
                            container
                            delay
                            dimension-mapping
                            dynamic 
                            envelope
                            expression
                            foreign-call
                            input
                            last-sample
                            midi
                            noise
                            null-controller
                            osc
                  	        random
                            scale
                            signal
                            sine
                            sound-file
                            spread
                            sum
														timer
                            variable-second-order-filter
                            ) 
         (const
          set-breakpoint
          ))
        ("Connections" nil ( adhere
                             bi-fingerboard 
                             bow
                             felt
                             force
                             hole
                             labium
                             mono-fingerboard
                             normalised-valve
                             pluck
                             position-connection
                             reed
                             strike
                             speed 
                             spring
                             valve) () nil)
        ("Input/Outputs" nil (access 
                              hybrid-access
                              point-input
                              point-output
                              tri-hybrid-access) () nil)
        ("Components" nil (key hammer modal-data bpf-bundle) () nil)
        ("Mesh" nil (mesh-add 
                     mesh-copy  
                     mesh-single-point
                     mesh-read-from-file
                     mesh-restrict-quadrilateral
                     mesh-restrict-plane) (mlys:duplicate mlys:transform mlys:mview) nil)
        ("Plots" nil (plot) (plot-value plot) nil)
        ))

;--------------------------------------------------
;filling packages
;--------------------------------------------------

(om::fill-library *subpackages-list*)

