;;;-*-Mode: Lisp; Package: OM -*-

(in-package :om)

;;;;;;;;;;;;;; funzioni per gli envelopes ;;;;;;;;;;;;;;

;============editor

(defclass mlys::enveditor (arrayeditor) 
  ((show-controls-p :initform t :accessor show-controls-p)))

(defmethod get-control-list ((self mlys::enveditor)) 
  (loop for item in (lcontrols (object self))
        collect (list (car item) t)))

(defmethod initialize-instance :after ((self mlys::enveditor) &rest l) 
  (declare (ignore l))
  (set-units-ruler (panel self) (rulerx (panel self))))

(defmethod add-some-controls ((self mlys::enveditor) view show-cont grupo) nil)

(defmethod editor-array-panel-class ((self mlys::enveditor)) 'mlys::envarraypanel)

(defmethod spline ((self mlys::enveditor))
  self)
(defmethod active ((self mlys::enveditor))
  self)
(defmethod compute-spline ((self mlys::enveditor))
  self)

;--------panel
(defclass mlys::envarraypanel (arraypanel) ())

(defmethod get-string-nom  ((self mlys::envarraypanel)  num (axe (eql 'x)))
    (format () "~,2F" (/ num (expt 10.0 6))))

(defmethod special-set-ranges ((self mlys::envarraypanel) new-pi-panel decimals rangebpf)
  (set-ranges new-pi-panel (rangex self) 
              (list (third rangebpf) (fourth rangebpf))))

;--------bpf
(defmethod bpf-panel-class ((self mlys::envarraypanel)) 'bpf-env-parameter-panel)

(defclass bpf-env-parameter-panel (bpf-parameter-panel) ())

(defmethod draw-grille  ((self bpf-env-parameter-panel))
   (with-focused-view self
     (draw&print-grille self :offx 0  :offy 0)))


;============================

(defclass! mlys::bpf-bundle (class-Array)
  ((mlys::times :initform '(0) :accessor mlys::times))
  (:icon 188 )
  (:documentation "
a class represention the modalys 'envelope controller. The dimension of this controller is given by the number of optional arguments.
Inputs (left to right)
self. a mlys::bpf-bundle object
times. a list of sorted timepoints starting from zero (the absolute times of the breakpoints)
dim1, dim2 etc. the values output by the controller at the given time."))

(defmethod get-slot-in-out-names ((self mlys::bpf-bundle))
   (values '("self" "times") '(nil (0))
           '("time list" "Sorted list of onsets [seconds]")
           '(nil nil )))

(defmethod get-editor-class ((self mlys::bpf-bundle))  'mlys::enveditor)

(defmethod get-type-of-ed-box ((self mlys::bpf-bundle))  'envBox)

(defmethod dimension ((self mlys::bpf-bundle))  
   (length (lcontrols self)))

(defmethod fixed-slots-list ((self mlys::bpf-bundle)) '(mlys::times))

(defmethod verify-list-exacte ((self list) type num (array mlys::bpf-bundle))
   (cond 
    ((subtypep type 'list)
     (if (and (>= (length self) num) (list-subtypep self type))
       (make-instance 'exactlist :thelist (first-n self num))
       self))
    ((subtypep type 'number)
     (if (and (>= (length self) num) (list-subtypep self type))
       (make-instance 'exactbpf 
         :thebpf (simple-bpf-from-list (mlys::times array) (first-n self num) 'bpf 6))
       self))
    (t self)))

(defmethod get-array-ranges ((self mlys::bpf-bundle))
   (list (round (* (car (mlys::times self)) 1000000)) (round (* 1000000 (car (last (mlys::times self)))))))

(defmethod get-bfp-for-draw ((self number) (array mlys::bpf-bundle) preci type)
   (simple-bpf-from-list (list (car (mlys::times array)) (car (last (mlys::times array)))) (list self self) 'bpf preci))

(defmethod get-bfp-for-draw ((self list) (array mlys::bpf-bundle) preci type)
   (if (list-subtypep self 'number)
     (simple-bpf-from-list (mlys::times array) (get-points self 'number array) 'bpf preci)
     nil))

(defmethod get-bfp-for-draw ((self function) (array mlys::bpf-bundle) preci type) 
   (simple-bpf-from-list (mlys::times array) (get-points self 'number array) 'bpf preci))

(defmethod cons-new-array ((self mlys::bpf-bundle) args argkeys objs class)
   (if objs
     (objFromObjs (first args) self)
     (let* ((rep (make-instance (type-of self) :numcols (length (second args))))
            (precision-list 6))
       (setf (mlys::times rep) (second args))
       (loop while argkeys do
             (cond
              ((string-equal (string (car argkeys)) "precision")
               (pop argkeys)
               (setf precision-list (pop argkeys)))
              ((array-special-keyword rep (car argkeys))
               (array-special-keyword-action rep (pop argkeys) (pop argkeys)))
              (t
               (pushr (list (pop argkeys) (verify-list-exacte (pop argkeys) 'number (numcols rep) rep)) (Lcontrols rep)))))
       (put-precision rep precision-list)
       rep)))

(defmethod draw-obj-in-rect ((self  mlys::bpf-bundle) x x1 y y1 edparams  view)
   (let* ((numvis (length (Lcontrols self)))
          (preci (get-precision self )) deltay deltax)
     (setf deltay (max 15 (round (- y1 y)  numvis))
           deltax (max 10 (round (- x1 x) (numcols self))))
            (loop for i in (Lcontrols self)
                  for k = 0 then (+ k 1)
                  for posy = y then (+ posy deltay) do
                  (let ((bpf? (get-bfp-for-draw  (second i) self preci 'number))) 
                    (if bpf?
                      (progn
                        (setf (bpfcolor bpf?) (nth k (color-list edparams)))
                        (draw-obj-in-rect  bpf? x x1  posy (+ posy deltay) (default-edition-params self) view))
                      (loop for di = 1 then (+ di 1)
                            for posx = (+ x (* di deltax))
                            while (< posx x1)  do
                            (draw-line posx posy posx (+ posy deltay))))
                    (draw-line x (+ posy deltay) x1  (+ posy deltay))
                    (draw-line x (+ 1 posy deltay) x1  (+ posy  1 deltay))))))

;copy and save

(defmethod omNG-copy ((self mlys::bpf-bundle))
   `(let ((copy ,(call-next-method)))
        (setf (mlys::times copy) ',(mlys::times self))
        copy))


;======================

(defclass envBox (arrayBox) ())


(defmethod do-add-one-keyword ((self envBox))
   (om-om-beep-msg "No keywords for enveloppes"))

(defmethod do-delete-one-keyword ((self envBox)) nil)


(defmethod do-add-one-input ((self envBox))
   (let* ((normalinputs (find-class-boxes  (inputs self) 'input-funbox))
          (paraminputs (find-class-boxes  (inputs self) 'env-keyword)))
     (setf (inputs self) (append normalinputs
                                 (append paraminputs
                                         (list (make-instance 'env-keyword
                                                 :name (format nil "Dim~D" (+ 1  (length paraminputs)))         
                                                 :box-ref self
                                                 :value (string2initarg (format nil "Dim~D" (+ 1  (length paraminputs)))) 
                                                 :doc-string "Extra dimension")))))))


(defmethod do-delete-one-input ((self envBox))
   (let* ((usedkeywords (find-class-boxes (inputs self) 'env-keyword)))
     (when usedkeywords
       (setf (inputs self) (remove (car (last usedkeywords)) (inputs self) :test 'equal)))))

(defmethod matrix-gen-code ((self envBox) (val mlys::bpf-bundle))
   (let* ((second-input (gen-code (second (inputs self)) nil))
          (params (decode self))
          (theval (if (connected? (first (inputs self)))
                    `(objFromObjs ,(gen-code (first (inputs self)) nil) ,val)
                    `(let* ((timeline ,second-input)
                            (err (error-test-times timeline)))
                       (when err (error err))
                       (funcall 'cons-new-array ,val (list nil timeline) (list ,.(cddr params)) nil ',(class-name (reference self)))))))
     theval))

(defmethod omNG-box-value ((self envBox) &optional (numout 0))
   (handler-bind ((error #'(lambda (c)
                             (when *msg-error-label-on*
                               (message-dialog (format nil "Error while evaluating the ~D factory ~%~D" (class-name (reference self)) 
                                                       (ccl::report-condition c nil))
                                               :size (make-point 300 200))
                               (abort)))))
     (let ((editorclass (class-name (reference self))))
       (cond
        ((equal (allow-lock self) "l") (special-lambda-value self editorclass))
        ((equal (allow-lock self) "o") (reference self))
        ((and (equal (allow-lock self) "x") (value self)) 
         (rep-editor (value self) (numout2label-num self numout)))
        ((and (equal (allow-lock self) "&") (ev-once-p self)) (rep-editor (value self) (numout2label-num self numout)))
        (t (let* ((args (loop for input in (inputs self)
                               when (not (keyword-input-p input)) collect (omNG-box-value  input)))
                  (argkeys (eval-keywords self))
                  (error (error-test-times (second args)))
                  rep)
             (when error (error error))
             (setf rep (cons-new-array (value self) args argkeys (connected? (first (inputs self))) editorclass))
             (if (null rep)
               (progn
                 (om-beep-msg (string+ "I can not construct a " (string editorclass) " with these parameters"))
                 (abort))
               (progn
                 (setf (value self) rep)
                 (when (showpict self)
                   (update-miniview (iconview (car (frames self))) (value self)))
                 (when (editorFrame self)
                   (update-editor-after-eval (editorFrame self) rep))
                 (when (equal (allow-lock self) "&")
                   (setf (ev-once-p self) t))
                 (numout2label-num self numout)
                 (rep-editor (value self) (numout2label-num self numout))))))))))


(defclass env-keyword (editor-keyword) ())

(defmethod eval-keyword-in ((self env-keyword))
   (declare (ignore numout))
   (list (value self) 
         (if (connected? self)
           (omNG-box-value (first (connected? self)) (second (connected? self)))
           (eval-not-connected self))))
   
(defmethod gen-code-keyword-in ((self env-keyword))
   (declare (ignore numout))
   (let ((theinput (connected? self))
         (value (value self)))
     (list value
           (if theinput
             (gen-code (first theinput) (second theinput))
             (code-not-connected self)))))

(defmethod eval-not-connected ((self env-keyword))
   (om-beep-msg (format nil "WARNING keyword ~D is not connected, NIL is used as default value." (string (value self)))))
   
(defmethod code-not-connected ((self env-keyword))
   (om-beep-msg (format nil "WARNING keyword ~D is not connected, NIL is used as default value." (string (value self)))))

(defun error-test-times (list)
  (cond
   ((null list) "Times can not be NIL")
   ((not (listp list)) "Times must be a list")
   ((not (= (car list) 0)) "First element of times must be zero")
   ((not (apply '< list)) "Times are out of order")
   (t nil)))
  
