;;;-*-Mode: Lisp; Package: Modalys -*-

(in-package :mlys)

(defun chords->single_notes (cseq)  
  (let* ((note (om::lmidic cseq))
         (ons (om::om/ (om::lonset cseq) 1000.0))
         (vel (om::lvel cseq))
         (dur (om::om/ (om::ldur cseq) 1000.0)))
    (om::flat 
     (mapcar #'(lambda (a b c d)
                 (if 
                   (= 1 (length a))
                   (list (list (car a) b (car c) (car d)))
                   (do ((a- a (cdr a-))
                        (c- c (cdr c-))
                        (d- d (cdr d-))
                        (lista '() (cons
                                    (list (car a-) b (car c-) (car d-))
                                    lista)))
                       ((endp a-) (reverse lista)))))
             note ons vel dur)
     1)))

(om::defmethod! chord-seq-spread ((cseq om::chord-seq))
  :icon 128
  :indoc '("a chord-seq obj")
  :doc "for each note of a chord-seq (here in frequency) it gives the relative onsets (in seconds), midivels and durations"
  (let* ((note (om::sort. (remove-duplicates (om::flat (om::lmidic cseq))) #'<))
         (liste (chords->single_notes cseq))
         (same-note (mapcar #'(lambda (pitch)
                                (cons (om::mc->f pitch)
                                      (cdr
                                       (om::mat-trans
                                        (remove-if-not
                                         #'(lambda (y)
                                             (= (car y)
                                                pitch)) liste))))) note)))
    same-note))

(om::defmethod! onsets->strike ((onsets list)
                            (offset float)
                            (attack-time float)
                            (release-time float)
                            (attack-pos float)
                            (release-pos float))
  :initvals '(nil 0.2 0.005 0.005 -0.01 0.1 'env0)
  :indoc '( "onsets (not milliseconds!!!)" "offset (it can't be = 0)" "attack-time" "release-time" "attack-position" "release-position" "name of the envelope")
  :icon 128
  :doc "onsets->strike build from a list of onsets the envelope for a hammer"
  
  (let* ((onsets+ (om::om+ offset onsets))
         (attack-ons (om::om- onsets+ attack-time))
         (attack-posns (make-list (length attack-ons) :initial-element release-pos))
         (release-ons (om::om+ onsets+ release-time))
         (release-posns (make-list (length release-ons) :initial-element release-pos))
         (gia-pos (make-list (length onsets+) :initial-element attack-pos))
         (alto1 (mapcar #'list attack-ons attack-posns))
         (gia (mapcar #'list onsets+ gia-pos))
         (alto2 (mapcar #'list release-ons release-posns))
         (inizio (list (list 0.0 release-pos)))
         (tutto (om::mat-trans (om::sort. (append inizio alto1 gia alto2) #'< #'car))))
    tutto))

