;;;-*-Mode: Lisp; Package: Modalys -*-
;;;;;;;;;; Mauro Lanza, IRCAM Gennaio 2000 ;;;;;;;;;;;;
;;;;;;;;;; ver 0.0 ;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package Modalys)

;;;;;;;;;;;; classe modal-dataect ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(om::defclass! modal-data ()
  ((obj-info :initform "" :accessor obj-info :initarg :obj-info :type string)
   (freqs :initform nil :accessor freqs :initarg :freqs :type list)
   (bwdth :initform nil :accessor bwdth :initarg :bwdth :type list)
   (deformees :initform nil :accessor deformees :initarg :deformees :type list))
  (:icon 128 )
  (:documentation "

modal-data is a class that contains the information issued from the modal data of a modalys object 
(saved in Modalys with the save-object function), from a resonance-model created by ResAn or from a 
coll-file for the MSP object resonators~ .

Inputs (left to right)

self. a modal-data object

obj-info. the string that allows Modalys to recognise an object and to make accesses on it. If it's 
          empty it means that the object is unknown, and so the only way to access it is by means of
          'decoded access

freqs. a list of frequencies

bwdth. a list of bandwidth

deformees. a list of lists of amplitudes ; each sublist contains the amplitude of a frequency 
(the one with the same index) for each point of the object (the so-called 'modal deformation'). 
So the length of each sublist is equal to the number of points the object (in case of a resonance 
model there will be just one point)."))

(defmethod om::get-type-of-ed-box ((self modal-data))  'ModalysObjBox)

(defclass ModalysObjBox (om::OMBoxEditCall) ())

(defmethod om::import-box-editor ((self ModalysObjBox))
   (catch-cancel
     (let ((name (CHOOSE-FILE-DIALOG)))
         (setf (value self) (read-object name)))))

(defmethod om::export-box-editor ((self ModalysObjBox))
   (catch-cancel
     (let ((name (choose-new-file-dialog    
                    :prompt "Save the modal-data here")))
         (write-object self name))))

;;;;;;;;;;;;; verifica se il file e un file .mos ;;;;;;;;;;;;;;;;

(defun verify-if-mod (string)
  (let ((lista (coerce string 'list)))
    (if (not 
         (equalp 
          (last-n lista 4)
          '(#\. #\m #\o #\s)))
         (coerce
          (append lista '(#\. #\m #\o #\s)) 'string)
         string)))

(defun rec-read-from-string (string)
  "utilissimo!"
  (labels ((fun (x)
             (multiple-value-list (read-from-string x nil))))
    (if (null (read-from-string string nil))
      nil
      (cons (car (fun string))
            (rec-read-from-string (coerce
                                   (nthcdr
                                    (cadr (fun string))
                                    (coerce string 'list))
                                   'string))))))


;;;;;;;;;; funzioni per leggere modelli di risonanza creati da ResAn ;;;;;;

(defun modres-p (file)
  (with-open-file (object-stream file
                                 :direction
                                 :input)
    (do ((linea (read-line object-stream nil) (read-line object-stream nil)))
        ((not (string= linea "")) 
         (if (string-equal "*** INFORMATION DEL MODELE ***" linea)
           t nil)))))

(defun initial-pos (stream)
  (let ((pos 0) rep)
    (file-position stream 0)
    (loop while (not (stream-eofp stream))
          while (not rep) do
          (setf pos (file-position stream))
          (let ((test (search  "model_size" (read-line stream))))
            (if (and test (zerop test))
              (setf rep t))))
    (if rep pos )))
          
;;;;;;;;;; funzioni per leggere file coll per l'oggetto max resonators~ ;;;;;;

(defun max-coll-p (file)
  (with-open-file (object-stream file
                                 :direction
                                 :input)
    (do ((linea (read-line object-stream nil) (read-line object-stream nil)))
        ((not (string= linea "")) 
         (if (and (or (char= (elt linea 0) #\1) (char= (elt linea 0) #\0))
                  (char= (elt linea 1) #\,)
                  (char= (elt linea 2) #\Space))
           t
           nil)))))

(defun read-max-coll (file)
  (let ((freqs '())
        (amps '())
        (bands '()))
    (progn
      (with-open-file (object-stream file
                                     :direction
                                     :input)
        (do ((linea (read-line object-stream nil) (read-line object-stream nil)))
            ((null linea))
          (let ((lista (rec-read-from-string 
                        (coerce 
                         (butlast 
                          (member #\Space
                                  (coerce linea 'list))) 
                         'string))))
            (progn (push (car lista) freqs)
                   (push (cadr lista) amps)
                   (push (caddr lista) bands)))))
      (make-instance 'modal-data 
      :freqs (reverse freqs)
      :bwdth (reverse bands)
      :deformees (mapcar #'list (reverse amps))))))
      

;;;;;;;;;; funzioni per leggere i file contenenti i dati modali ;;;;;

(defun get-obj-info (modal-file)
  (with-open-file (object-stream modal-file
                                 :direction
                                 :input
#+mcl                            :external-format
#+mcl                             :unix
)
    (do ((i 0 (1+ i))
         (linea nil (read-line object-stream nil)))
        ((= 1 i) linea))))

(defun freqs&bwdth (modal-file)
  (let* ((inizia? '())
         (liste '()))
   (progn
    (with-open-file (object-stream modal-file
                                   :direction
                                   :input
#+mcl                              :external-format
#+mcl                              :unix
)
      (do ((linea nil (read-line object-stream nil)))
          ((search "mode0:" linea))
        (cond ((search "number_of_points:" linea) (push 'vai!!! inizia?))
              ((not (null inizia?))
               (let* ((lista-char (coerce linea 'list))
                      (lista-char2 (remove-if-not #'(lambda (x)
                                                      (or (char-equal x #\Space)
                                                          (char-equal x #\.)
                                                          (digit-char-p x))) 
                                                  lista-char))
                      (string-new (coerce lista-char2 'string))
                      (fine (rec-read-from-string string-new)))
                 (push (list (second fine)
                             (fourth fine))
                       liste))))))
      (om::mat-trans (reverse liste)))))

(defun get-deformees (modal-file)
  (let ((start '())
        (deform-seq '()))
    
    (with-open-file (object-stream modal-file
                                   :direction
                                   :input
#+mcl                              :external-format
#+mcl                              :unix
)
      (do ((linea nil (read-line object-stream nil))
           (deform nil (cond 
                        ((string-equal linea "mode0:") 
                         (progn 
                           (push 'vai!!! start) 
                           nil))
                        ((endp start) nil)
                        ((search "mode" linea) 
                         (progn
                           (push (reverse deform)
                                 deform-seq)
                           nil))
                        (t (cons (read-from-string linea nil)
                                 deform)))))
          ((and (not (endp start))
                (null linea))
           (reverse
            (push (reverse deform)
                  deform-seq)))))))
                 
(defun read-modal-data (file)
  (let* ((info (get-obj-info file))
         (f&b (freqs&bwdth file))
         (frequenze (car f&b))
         (bandwidth (cadr f&b))
         (deformate (get-deformees file)))
    (make-instance 'modal-data 
      :obj-info info
      :freqs frequenze
      :bwdth bandwidth
      :deformees deformate)))

(defmethod read-object ((name pathname))
    (cond ((max-coll-p name) (read-max-coll name))
          ((modres-p name) (read-modres name))
          (t (read-modal-data name))))
    
                    
(om::defmethod! for-MSP-resonators~ ((obj modal-data) 
                                     (point integer))
  :icon 129
  :indoc '("a modalys object" "the index of a point on the modalys object")
  :doc "Format the data stored in the modal-data class as a coll for the MSP object resonators~. 
Since resonators is a resonance model it has no physical dimensions (it's something like a 'point'); 
so if you want to play in real-time with a modalys object you have to select the index of the point you are going to listen to."  
  (let* ((name (choose-new-file-dialog 
                                :prompt "coll file for max resonators~"
                                :button-string "coll-file"))
         (frequenze (freqs obj))
         (amplitudes (deformees obj))
         (bands (bwdth obj))
         (n-modes (length frequenze)))
    (with-open-file (outstream name
                               :direction
                               :output)
      (do ((i 1 (1+ i)))
          ((= i (1+ n-modes)))
        (format outstream "~&~S, ~S ~S ~S;~%" 
                i 
                (nth (1- i) frequenze)
                (nth point (nth (1- i) amplitudes))
                (nth (1- i) bands))))
    (set-mac-file-creator name :max2)))

(defmethod write-object ((self ModalysObjBox) (name pathname))
  (let ((final-name (if name name (get-temporary-file "modal"))))
    (with-open-file (outstream final-name
                               :direction
                               :output
#+mcl                          :external-format
#+mcl                          :unix
)
      (let* ((obj (value self))
             (info (obj-info obj))
             (frequenze (freqs obj))
             (bands (bwdth obj))
             (amplitudes (deformees obj))
             (num-modes (length frequenze))
             (num-points (length (car amplitudes))))
        (progn
          (format outstream info)
          (format outstream "~%~%")
          (format outstream "~%Modal_data_for_Modalys_object:~%~%")
          (format outstream "number_of_modes: ~S~%" num-modes)
          (format outstream "number_of_points: ~S~%~%" num-points)
          (dotimes (i num-modes)
            (format outstream "freq~S_Hz:  ~S  abs~S_1/s:  ~S~%" i (nth i frequenze) i (nth i bands)))
          (dotimes (i num-modes)
            (progn
              (format outstream "mode~S:~%" i)
              (mapcar #'(lambda (x)
                          (format outstream "~S~%" x))
                      (nth i amplitudes)))))))
    final-name))

(defmethod write-object ((self modal-data) (name pathname))
    (with-open-file (outstream name
                               :direction
                               :output
#+mcl                          :external-format
#+mcl                          :unix
)
      (let* ((obj self)
             (info (obj-info obj))
             (frequenze (freqs obj))
             (bands (bwdth obj))
             (amplitudes (deformees obj))
             (num-modes (length frequenze))
             (num-points (length (car amplitudes))))
        (progn
          (format outstream info)
          (format outstream "~%~%")
          (format outstream "~%Modal_data_for_Modalys_object:~%~%")
          (format outstream "number_of_modes: ~S~%" num-modes)
          (format outstream "number_of_points: ~S~%~%" num-points)
          (dotimes (i num-modes)
            (format outstream "freq~S_Hz:  ~S  abs~S_1/s:  ~S~%" i (nth i frequenze) i (nth i bands)))
          (dotimes (i num-modes)
            (progn
              (format outstream "mode~S:~%" i)
              (mapcar #'(lambda (x)
                          (format outstream "~S~%" x))
                      (nth i amplitudes)))))))
    name)

