;;;-*-Mode: Lisp; Package: MODALYS -*-

(in-package "MODALYS")

(defpackage "MODALYS" 
  (:nicknames "MLYS")
  (:export 
   "SYNTHESIZE"))

(om::defclass! Modalys-Object ()
  ((private-ptr  :type macptr  :accessor private-ptr :initform (make-null-ptr))
   (maker        :type function :initform (lambda ()) :allocation :class :accessor maker)
   (ignore       :type integer :allocation :class :initform 1))
  (:documentation "Root object"))

(defmethod is-Modalys-Object ((x Modalys-Object)) t)
(defmethod is-Modalys-Object ((x t)) nil)

; c'est modalys si un des parent est modalys

(defvar *modalys-class* nil)
(setq *modalys-class* (class-of (make-instance 'modalys-object)))

(defun is-modalys (c)
  (or (eq c *modalys-class*)
      (loop for x in (class-direct-superclasses c)
            do (if (is-modalys x) (return t)))))

(defun allow (class-symbol slot-name predicate)
#+mcl  (if (>= om::*om-version* 5.01)
#+mcl    (let ((l (class-instance-slots (find-class class-symbol))))
#+mcl      (setf 
#+mcl       (slot-value (nth (position slot-name (mapcar (lambda (x) (slot-value x 'ccl::name)) l)) l)
#+mcl                   'ccl::type-predicate)
#+mcl       predicate)))
)

(setf (symbol-function 'modify-arglist-orig) (function modify-arglist))
(defun modify-arglist (arg)
  (let ((modified-args (modify-arglist-orig arg))
        (lmbd          (lambda (x) 
                         (if (is-Modalys-Object x)
                           (slot-value x 'private-ptr)
                           (if (listp x)
                             (modify-arglist x)
                             x)))))
    (if (listp modified-args)
      (mapcar lmbd modified-args)
      (funcall lmbd modified-args))))

(om::defclass! Controller (Modalys-Object)
  ((dimension    :initform 1      :type number :accessor dimension :initarg :dimension)
   (maker        :initform (function modalys::make-controller) :type function :allocation :class))
  (:icon 564)
  (:documentation "Controller"))
(allow 'Controller 'dimension (lambda (x) (or (null x) (listp x) (numberp x))))

(om::defclass! Arithmetic (Controller)
  ((operation          :initform nil  :type symbol  :accessor operation         :initarg :operation)
   (controllers        :initform nil  :type list    :accessor controllers       :initarg :controllers))
  (:icon 535))
(setf (documentation (find-class 'Arithmetic) 'type) (get-documentation 'controllers 'Arithmetic))

(om::defclass! Bandlimited-Noise (Controller)
  ((update-interval    :initform 0  :type number  :accessor update-interval :initarg :update-interval)
   (cutoff             :initform nil  :type Controller  :accessor cutoff          :initarg :cutoff)
   (taps               :initform 4  :type integer :accessor taps            :initarg :taps))
  (:icon 536))
(setf (documentation (find-class 'Bandlimited-Noise) 'type) (get-documentation 'controllers 'Bandlimited-Noise))

(om::defclass! Constant (Controller)
  ((value        :initform nil    :type list    :accessor value    :initarg :value)))
(setf (documentation (find-class 'Constant) 'type) (get-documentation 'controllers 'Constant))
(allow 'Constant 'value (lambda (x) (or (null x) (listp x) (numberp x))))

(om::defclass! Constant-Second-Order-Filter (Controller)
  ((update-interval    :initform 0    :type number          :accessor update-interval :initarg :update-interval)
   (filter             :initform nil  :type Controller      :accessor filter          :initarg :filter)
   (amplitudes         :initform nil  :type list            :accessor amplitudes      :initarg :amplitudes)
   (centers            :initform nil  :type list            :accessor centers         :initarg :centers)
   (bandwidths          :initform nil :type list            :accessor bandwidths      :initarg :bandwidths))
  (:icon 537))
(setf (documentation (find-class 'Constant-Second-Order-Filter) 'type) (get-documentation 'controllers 'Constant-Second-Order-Filter))

(om::defclass! Container (Controller)
  ((contained   :initform nil  :type Controller  :accessor contained   :initarg :contained)))
(setf (documentation (find-class 'Container) 'type) (get-documentation 'controllers 'Container))



(om::defclass! Dimension-Mapping (Controller)
  ((controllers        :initform nil  :type list            :accessor controllers     :initarg :controllers))
  (:icon 538))
(setf (documentation (find-class 'Dimension-Mapping) 'type) (get-documentation 'controllers 'Dimension-Mapping))

(om::defclass! Dynamic (Controller)
  ((update_interval    :initform -1      :type number :accessor update_interval :initarg :update_interval)
   (value        :initform nil    :type list    :accessor value    :initarg :value)
   (name         :initform ""    :type string  :accessor name     :initarg :name))
  (:icon 517))
(setf (documentation (find-class 'Dynamic) 'type) (get-documentation 'controllers 'Dynamic))
(allow 'Dynamic 'name (lambda (x) (or (stringp x) (null x))))
(allow 'Dynamic 'value (lambda (x) (or (listp x) (numberp x) (null x))))
(allow 'Dynamic 'dimension (lambda (x) (or (null x) (listp x) (numberp x))))

(om::defclass! Delay (Controller)
  ((delta-time  :initform  nil  :type Controller  :accessor delta-time :initarg :delta-time)
   (to-follow    :initform nil  :type Controller  :accessor to-follow   :initarg :to-follow))
	(:icon 538))
(setf (documentation (find-class 'Delay) 'type) (get-documentation 'controllers 'Delay))

(om::defclass! Envelope (Controller)
  ((value        :initform nil  :type list :accessor value    :initarg :value))
  (:documentation "A break-point envelope controller")
  (:icon 503))
(setf (documentation (find-class 'Envelope) 'type) (get-documentation 'controllers 'Envelope))
(allow 'Envelope 'value (lambda (x) (or (null x)(number-matrixable x)(typep x 'om::bpf))))

(om::defclass! Expression (Controller)
  ((update-interval    :initform 0  :type number :accessor update-interval    :initarg :update-interval)
   (string-function    :initform ""  :type string :accessor string-function :initarg :string-function)
   (controllers        :initform nil  :type list    :accessor controllers       :initarg :controllers))
  (:documentation "Allows to have a lambda function beeing evaluated during the synthesis")
  (:icon 575))
(allow 'Expression 'controllers (lambda (x) (or (null x)(numberp x)(listp x)(typep x 'Controller))))

(om::defclass! Foreign-Call (Controller)
  ((update-interval    :initform 0  :type number :accessor update-interval    :initarg :update-interval)
   (foreign-function   :initform (lambda (x y))  :type function :accessor foreign-function :initarg :foreign-function)
   (controllers        :initform nil  :type list    :accessor controllers       :initarg :controllers))
  (:documentation "Allows to have a lambda function beeing evaluated during the synthesis")
  (:icon 523))
(allow 'Foreign-Call 'foreign-function (lambda (x) (or (null x) (functionp x))))

(om::defclass! Input (Controller)
  ((channel    :initform 0  :type number          :accessor channel :initarg :channel))
  (:icon 566))

(om::defclass! Last-Sample (Controller)
  ()
  (:icon 539))
(setf (documentation (find-class 'Last-Sample) 'type) (get-documentation 'controllers 'Last-Sample))

(om::defclass! Midi (Controller)
  ((file-name       :initform ""  :type string :accessor file-name    :initarg :file-name)
   (channel         :initform 0    :type number :accessor channel      :initarg :channel)
   (data-types      :initform nil  :type list   :accessor data-types   :initarg :data-types))
  (:icon 529))
(setf (documentation (find-class 'Midi) 'type) (get-documentation 'controllers 'Midi))

(om::defclass! Noise (Controller)
  ((update-interval    :initform 0    :type number          :accessor update-interval :initarg :update-interval)
   (increment          :initform nil  :type Controller      :accessor increment       :initarg :increment)
   (xrmin              :initform nil  :type Controller      :accessor xrmin           :initarg :xrmin)
   (xrmax              :initform nil  :type Controller      :accessor xrmax           :initarg :xrmax)
   (yrmin              :initform nil  :type Controller      :accessor yrmin           :initarg :yrmin)
   (yrmax              :initform nil  :type Controller      :accessor yrmax           :initarg :yrmax)
   (taps               :initform 0    :type integer         :accessor taps            :initarg :taps))
  (:icon 540))
(setf (documentation (find-class 'Noise) 'type) (get-documentation 'controllers 'Noise))

(om::defclass! Osc (Controller)
  ((initial-value         :initform nil  :type list      :accessor initial-value           :initarg :initial-value)
   (message               :initform ""  :type string    :accessor message                 :initarg :message))
  (:icon 563))
(allow 'OSC 'initial-value (lambda (x) (or (numberp x)(listp x))))

(oa::om-with-redefinitions 
(om::defclass! Random (Controller)
  ((update-interval    :initform 0  :type number          :accessor update-interval :initarg :update-interval))
  (:icon 541))
)
(setf (documentation (find-class 'Random) 'type) (get-documentation 'controllers 'Random))

(om::defclass! Scale (Controller)
  ((old-range-low      :initform 0  :type number :accessor old-range-low    :initarg :old-range-low)
   (old-range-high     :initform 0  :type number :accessor old-range-high   :initarg :old-range-high)
   (new-range-low      :initform 0  :type number :accessor new-range-low    :initarg :new-range-low)
   (new-range-high     :initform 0  :type number :accessor new-range-high   :initarg :new-range-high)
   (controller         :initform nil  :type Controller   :accessor controller       :initarg :controller))
  (:icon 522))
(setf (documentation (find-class 'Scale) 'type) (get-documentation 'controllers 'Scale))
(shadow 'signal :mlys)
(om::defclass! mlys::Signal (mlys::Controller)
  ((inputs        :initform nil    :accessor inputs       :initarg :inputs))
  (:icon 567))
(setf (documentation (find-class 'Signal) 'type) (get-documentation 'controllers 'Signal))

(om::defclass! Sine (Controller)
  ((controller        :initform nil  :type Controller    :accessor controller       :initarg :controller))
  (:icon 524))
(setf (documentation (find-class 'Sine) 'type) (get-documentation 'controllers 'Sine))

(om::defclass! Sound-File (Controller)
  ((update-interval    :initform 0  :type number          :accessor update-interval :initarg :update-interval)
   (sample-rate        :initform nil  :type Controller    :accessor sample-rate     :initarg :sample-rate)
   (file-name          :initform ""  :type string         :accessor file-name       :initarg :file-name)
   (offset             :initform 0  :type number          :accessor offset          :initarg :offset)
   (duration           :initform 0  :type number          :accessor duration        :initarg :duration)
   (taps               :initform 4  :type integer         :accessor taps            :initarg :taps))
  (:icon 568))
(setf (documentation (find-class 'Sound-File) 'type) (get-documentation 'controllers 'Sound-File))

(om::defclass! Spread (Controller)
  ((update-interval    :initform 0    :type number          :accessor update-interval :initarg :update-interval)
   (controller         :initform nil  :type list            :accessor Controller      :initarg :controller))
  (:icon 543))
(setf (documentation (find-class 'Spread) 'type) (get-documentation 'controllers 'Spread))

(om::defclass! Sum (Controller)
  ((update-interval    :initform 0    :type number          :accessor update-interval :initarg :update-interval)
   (controller         :initform nil  :type list            :accessor Controller      :initarg :controller))
  (:icon 544))
(setf (documentation (find-class 'Sum) 'type) (get-documentation 'controllers 'Sum))

(om::defclass! Timer (Controller)
  ((interval        :initform nil  :type Controller    :accessor interval       	:initarg :interval)
  (auto-reload	:initform nil  :type Controller    :accessor auto-reload	:initarg :auto-reload))
  )
(setf (documentation (find-class 'Timer) 'type) (get-documentation 'controllers 'Timer))


(om::defclass! Variable-Second-Order-Filter (Controller)
  ((update-interval    :initform 0    :type number          :accessor update-interval :initarg :update-interval)
   (input              :initform nil  :type Controller      :accessor input           :initarg :input)
   (center             :initform nil  :type Controller      :accessor center          :initarg :center)
   (bandwidth          :initform nil  :type Controller      :accessor bandwidth       :initarg :bandwidth))
  (:icon 545))
(setf (documentation (find-class 'Variable-Second-Order-Filter) 'type) (get-documentation 'controllers 'Variable-Second-Order-Filter))

(om::defclass! Mesh (Modalys-Object)
  ((maker              :initform (function modalys::make-mesh) :allocation :class)
   (which              :initform nil   :type symbol :accessor which  )
   (ignore            :allocation :class :initform 2))
  (:documentation "Generic Mesh")
  (:icon 554))

(om::defclass! Mesh-Single-Point (Mesh)
  ((which    :initform 'single-point     :type symbol :accessor which  )
   (point       :initform nil    :type vector   :accessor point   :initarg :point))
  (:documentation "single point mesh")
  (:icon 557))

(om::defclass! Mesh-Add (Mesh)
  ((which    :initform 'add     :type symbol :accessor which  )
   (meshes       :initform nil    :type list   :accessor meshes   :initarg :meshes))
  (:documentation "mesh resulting of the addition of several mershes")
  (:icon 556))

(om::defclass! Mesh-Copy (Mesh)
  ((which       :initform 'copy     :type symbol :accessor which )
   (original    :initform nil       :type Mesh   :accessor original   :initarg :original))
  (:documentation "mesh obtained by duplication")
  (:icon 555))

(om::defclass! Mesh-Read-From-File (Mesh)
  ((which    :initform 'read-from-file     :type symbol :accessor which  )
   (file-name    :initform nil  :type string :accessor file-name  :initarg :file-name))
  (:documentation "mesh obtained by duplication")
  (:icon 559))

(om::defclass! Mesh-Restrict-Quadrilateral (Mesh)
  ((which    :initform 'restrict-quadrilateral     :type symbol :accessor which  )
   (original        :initform nil    :type Mesh   :accessor original      :initarg :original)
   (quadrilateral   :initform '#(0 0 0 0)    :type vector :accessor quadrilateral :initarg :quadrilateral))
   (:documentation "mesh obtained by restriction to certain quadrilaterals")
   (:icon 558))

(om::defclass! Mesh-Restrict-Plane (Mesh)
  ((which    :initform 'restrict-plane     :type symbol :accessor which  )
   (original      :initform nil    :type Mesh   :accessor original      :initarg :original)
   (generator     :initform '#(0 0 0)    :type vector :accessor generator     :initarg :generator)
   (translator    :initform '#(0 0 0)    :type vector :accessor translator    :initarg :translator))
   (:documentation "mesh obtained by restriction to a certain plane, defined by its generator (any normal vector to the plane) and its translator (any vector belonging to the plane)")
   (:icon 600))




(om::defclass! Plot (Modalys-Object)
  ((maker              :initform (function modalys::make-plot) :allocation :class)
   (which              :initform nil   :type symbol :accessor which  )
   (ignore             :allocation :class :initform 2))
  (:documentation "Generic Plot")
  (:icon 0))











(defmacro make-get-slot-in-out-names (class_input)
  (let*
    (
     (cls     (if (not (symbolp class_input)) (eval class_input) class_input))
     (clsym   (class-name cls))
     (name-getter (if (> om::*om-version* 5.0) (lambda (x) (clos:slot-definition-name x)) (function car)))
     (initform-getter (if (> om::*om-version* 5.0) (lambda (x) (clos:slot-definition-initform x)) (function third)))
     (ign (or 1 (funcall initform-getter (car (last (class-slots cls))))))
     (clis_fa (om-api:class-instance-slots cls))
     (clis (last clis_fa (- (length clis_fa) ign)))
     (clname (symbol-name clsym))
     (n (get-num-allowed-pitch-parameter clname))
     (restrictor (if (eq 0 n) (lambda (x) (cddr x)) (lambda (x) x)))
     (names
      (cons "self"
            (funcall restrictor
                     (loop 
                       for slot in (mapcar name-getter clis)
                       collect (string-downcase (symbol-name slot))))))
     (defaults
       (cons nil
             (funcall restrictor
                      (loop 
                        for slot in (mapcar initform-getter clis)
                        collect slot))))
     (docs
      (cons nil
            (funcall restrictor
                     (loop 
                       for slot in (mapcar initform-getter clis)
                       collect nil))))
     (menus
      (cons nil
            (funcall restrictor
                     (cons 
                      (list (cons 1 (list (cons (list "none" nil) (loop for i from 0 to (- n 1)
                                                                        collect 
                                                (let ((s  (mlys::get-allowed-pitch-parameter clname i)))
                                                  (cons s (list (list 'quote (find-symbol (string-upcase s) :mlys))))))))))
                      (cdr
                       (loop 
                         for slot in (mapcar name-getter clis)
                         collect nil)))))))
    `(defmethod om::get-slot-in-out-names ((self ,clsym))
       (values 
        ',names        ;;; noms des slots
        ',defaults     ;;; valeur par defaut
        ',docs         ;;; doc
        ',menus))))

(om::defclass! Simple-Object (Modalys-Object)
  ((maker        :type function :initform (function modalys::make-object-basic)  :allocation :class :accessor maker)
   (tuning-param :initform nil :type symbol      :accessor tuning-param :initarg :tuning-param)
   (tuning-freq  :initform nil  :accessor tuning-freq  :initarg :tuning-freq)
   (compiled-access :initform nil :type list     :accessor compiled-access :initarg :compiled-access)
   (modes-num    :initform 40  :type Controller   :accessor modes-num    :initarg :modes-num))
  (:documentation "Simple Object"))

(defmethod is-Simple-Object ((x Simple-Object)) t)
(defmethod is-Simple-Object ((x t)) nil)

(om::defclass! Wave-Guide (Modalys-Object)
  ((maker        :type function :initform (function modalys::make-object-basic)  :allocation :class :accessor maker)
   (tuning-param :initform nil :type symbol      :accessor tuning-param :initarg :tuning-param)
   (tuning-freq  :initform nil  :accessor tuning-freq  :initarg :tuning-freq)
   (compiled-access :initform nil :type list     :accessor compiled-access :initarg :compiled-access))
  (:documentation "Wave Guide"))


(om::defclass! Bi-String (Simple-Object)
  ((string-length  :initform (cadr (nth 1 mlys::default-string))   :accessor string-length :initarg :string-length )
   (tension        :initform (cadr (nth 2 mlys::default-string))   :accessor tension :initarg :tension )
   (density        :initform (cadr (nth 3 mlys::default-string))   :accessor density :initarg :density )
   (radius         :initform (cadr (nth 4 mlys::default-string))   :accessor radius :initarg :radius )
   (young          :initform (cadr (nth 5 mlys::default-string))   :accessor young  :initarg :young  )
   (freq-loss      :initform (cadr (nth 6 mlys::default-string))   :accessor freq-loss  :initarg :freq-loss  )
   (const-loss     :initform (cadr (nth 7 mlys::default-string))   :accessor const-loss :initarg :const-loss ))
  (:documentation "Any string or rod that vibrates in two transverse directions. " )
  (:icon 528))

(setf (documentation (find-class 'Bi-String) 'type) (get-documentation 'objects 'bi-string))
(make-get-slot-in-out-names (find-class 'Bi-String))

(om::defclass! Bi-Two-Mass (Simple-Object)
  ((small-mass   :initform (cadr (nth 0 mlys::default-bitwomass))  :accessor small-mass :initarg :small-mass)
   (large-mass   :initform (cadr (nth 1 mlys::default-bitwomass))  :accessor large-mass :initarg :large-mass)
   (stiffness1   :initform (cadr (nth 2 mlys::default-bitwomass))  :accessor stiffness1 :initarg :stiffness1)
   (stiffness2   :initform (cadr (nth 3 mlys::default-bitwomass))  :accessor stiffness2 :initarg :stiffness2)
   (freq-loss1   :initform (cadr (nth 4 mlys::default-bitwomass))  :accessor freq-loss1  :initarg :freq-loss1)
   (freq-loss2   :initform (cadr (nth 5 mlys::default-bitwomass))  :accessor freq-loss2  :initarg :freq-loss2)
   (const-loss1  :initform (cadr (nth 6 mlys::default-bitwomass))  :accessor const-loss1 :initarg :const-loss1)
   (const-loss2  :initform (cadr (nth 7 mlys::default-bitwomass))  :accessor const-loss2 :initarg :const-loss2)
   (ignore       :allocation :class :initform 2))
  (:documentation "A simple two-mass model that can serve as a hammer, plectrum, finger, etc.")
  (:icon 501))

(setf (documentation (find-class 'Bi-Two-Mass) 'type) (get-documentation 'objects 'Bi-Two-Mass))
(make-get-slot-in-out-names (find-class 'Bi-Two-Mass))

(om::defclass! Cello-Bridge (Simple-Object)
  ((m1-mass        :initform (cadr (nth 0 mlys::default-celbridge))  :accessor m1-mass :initarg :m1-mass)
   (m2-mass        :initform (cadr (nth 1 mlys::default-celbridge))  :accessor m2-mass :initarg :m2-mass)
   (m12-mass       :initform (cadr (nth 2 mlys::default-celbridge))  :accessor m12-mass :initarg :m12-mass)
   (d-distance     :initform (cadr (nth 3 mlys::default-celbridge))  :accessor d-distance :initarg :d-distance) 
   (d20-distance   :initform (cadr (nth 4 mlys::default-celbridge))  :accessor d20-distance :initarg :d20-distance)
   (h1-distance    :initform (cadr (nth 5 mlys::default-celbridge))  :accessor h1-distance :initarg :h1-distance)
   (a-distance     :initform (cadr (nth 6 mlys::default-celbridge))  :accessor a-distance :initarg :a-distance) 
   (i20-distance   :initform (cadr (nth 7 mlys::default-celbridge))  :accessor i20-distance :initarg :i20-distance)
   (d2-stiffness   :initform (cadr (nth 8 mlys::default-celbridge))  :accessor d2-stiffness :initarg :d2-stiffness)
   (s1-stiffness   :initform (cadr (nth 9 mlys::default-celbridge)) :accessor s1-stiffness :initarg :s1-stiffness)
   (freq-loss      :initform (cadr (nth 10 mlys::default-celbridge)) :accessor freq-loss  :initarg :freq-loss)
   (const-loss     :initform (cadr (nth 11 mlys::default-celbridge)) :accessor const-loss :initarg :const-loss)
   (ignore       :allocation :class :initform 2))
  (:documentation "A Cello bridge")
  (:icon 546))
(setf (documentation (find-class 'Cello-Bridge) 'type) (get-documentation 'objects 'Cello-Bridge))
;(make-get-slot-in-out-names (find-class 'Cello-Bridge))

(om::defclass! Circ-Object (Simple-Object)
  ())

(om::defclass! Circ-Membrane (Circ-Object)
  ((radius           :initform  (cadr (nth 1 mlys::default-circmem))  :accessor radius         :initarg :radius)
   (tension          :initform  (cadr (nth 2 mlys::default-circmem))  :accessor tension        :initarg :tension)
   (surface-density  :initform  (cadr (nth 3 mlys::default-circmem))  :accessor surface-density  :initarg :surface-density)
   (freq-loss        :initform  (cadr (nth 4 mlys::default-circmem))  :accessor freq-loss      :initarg :freq-loss)
   (const-loss       :initform  (cadr (nth 5 mlys::default-circmem))  :accessor const-loss     :initarg :const-loss))
  (:documentation "A circular membrane")
  (:icon 519))
(setf (documentation (find-class 'Circ-Membrane) 'type) (get-documentation 'objects 'Circ-Membrane))
(make-get-slot-in-out-names (find-class 'Circ-Membrane))

(om::defclass! Clamped-Circ-Plate (Circ-Object)
  ((radius       :initform (cadr (nth 1 mlys::default-circplate))   :accessor radius     :initarg :radius)
   (thickness    :initform (cadr (nth 2 mlys::default-circplate))   :accessor thickness  :initarg :thickness)
   (density      :initform (cadr (nth 3 mlys::default-circplate))   :accessor density    :initarg :density)
   (young        :initform (cadr (nth 4 mlys::default-circplate))   :accessor young      :initarg :young)
   (poisson      :initform (cadr (nth 5 mlys::default-circplate))   :accessor poisson    :initarg :poisson)
   (freq-loss    :initform (cadr (nth 6 mlys::default-circplate))   :accessor freq-loss  :initarg :freq-loss)
   (const-loss   :initform (cadr (nth 7 mlys::default-circplate))   :accessor const-loss :initarg :const-loss))
  (:documentation "")
  (:icon 534))
(setf (documentation (find-class 'Clamped-Circ-Plate) 'type) (get-documentation 'objects 'Clamped-Circ-Plate))
(make-get-slot-in-out-names (find-class 'Clamped-Circ-Plate))

(om::defclass! Clone (Simple-Object)
  ((original       :initform nil    :type Simple-Object :accessor original :initarg :original)
      (ignore       :allocation :class :initform 2))
  (:documentation "A clone")
  (:icon 547))
(setf (documentation (find-class 'Clone) 'type) (get-documentation 'objects 'Clone))
(make-get-slot-in-out-names (find-class 'Clone))

(om::defclass! Closed-Closed-Tube (Simple-Object)
  ((tube-length      :initform  (cadr (nth 1 mlys::default-tube))  :accessor tube-length    :initarg :tube-length )
   (air-elasticity   :initform  (cadr (nth 2 mlys::default-tube))  :accessor air-elasticity :initarg :air-elasticity )
   (air-density      :initform  (cadr (nth 3 mlys::default-tube))  :accessor air-density    :initarg :air-density )
   (radius0          :initform  (cadr (nth 4 mlys::default-tube))  :accessor radius0        :initarg :radius0 )
   (radius1          :initform  (cadr (nth 5 mlys::default-tube))  :accessor radius1        :initarg :radius1  )
   (freq-loss        :initform  (cadr (nth 6 mlys::default-tube))  :accessor freq-loss      :initarg :freq-loss  )
   (const-loss       :initform  (cadr (nth 7 mlys::default-tube))  :accessor const-loss     :initarg :const-loss ))
  (:documentation "An acoustic tube sealed at both ends")
  (:icon 548))
(setf (documentation (find-class 'Closed-Closed-Tube) 'type) (get-documentation 'objects 'Closed-Closed-Tube))
(make-get-slot-in-out-names (find-class 'Closed-Closed-Tube))

(om::defclass! Closed-Open-Tube (Simple-Object)
  ((tube-length      :initform  (cadr (nth 1 mlys::default-tube))  :accessor tube-length    :initarg :tube-length )
   (air-elasticity   :initform  (cadr (nth 2 mlys::default-tube))  :accessor air-elasticity :initarg :air-elasticity )
   (air-density      :initform  (cadr (nth 3 mlys::default-tube))  :accessor air-density    :initarg :air-density )
   (radius0          :initform  (cadr (nth 4 mlys::default-tube))  :accessor radius0        :initarg :radius0 )
   (radius1          :initform  (cadr (nth 5 mlys::default-tube))  :accessor radius1        :initarg :radius1  )
   (freq-loss        :initform  (cadr (nth 6 mlys::default-tube))  :accessor freq-loss      :initarg :freq-loss  )
   (const-loss       :initform  (cadr (nth 7 mlys::default-tube))  :accessor const-loss     :initarg :const-loss ))
  (:documentation "A closed open tube")
  (:icon 514))
(setf (documentation (find-class 'Closed-Open-Tube) 'type) (get-documentation 'objects 'Closed-Open-Tube))
(make-get-slot-in-out-names (find-class 'Closed-Open-Tube))


(om::defclass! Harmonic-Oscillator (Simple-Object)
  ((mass             :initform  (cadr (nth 0 mlys::default-harmonic-oscillator)) :accessor mass      :initarg :mass)
   (stiffness        :initform  (cadr (nth 1 mlys::default-harmonic-oscillator)) :accessor stiffness :initarg :stiffness)
   (freq-loss        :initform  (cadr (nth 2 mlys::default-harmonic-oscillator)) :accessor freq-loss :initarg :freq-loss)
   (const-loss       :initform  (cadr (nth 3 mlys::default-harmonic-oscillator)) :accessor const-loss :initarg :const-loss)
   (ignore       :allocation :class :initform 2))
  (:documentation "A harmonic oscillator. ")
  (:icon 579)
)
(setf (documentation (find-class 'Harmonic-Oscillator) 'type) (get-documentation 'objects 'Harmonic-Oscillator))
(make-get-slot-in-out-names (find-class 'Harmonic-Oscillator))

(om::defclass! Jet (Wave-Guide)
  ((pressure         :initform  (cadr (nth 0 mlys::default-jet)) :accessor pressure      :initarg :pressure)
   (air-density      :initform  (cadr (nth 1 mlys::default-jet)) :accessor air-density   :initarg :air-density)
   (celerity         :initform  (cadr (nth 2 mlys::default-jet)) :accessor celerity      :initarg :celerity)
   (jet-length       :initform  (cadr (nth 3 mlys::default-jet)) :accessor jet-length    :initarg :jet-length)
   (width            :initform  (cadr (nth 4 mlys::default-jet)) :accessor width         :initarg :width)
   (height           :initform  (cadr (nth 5 mlys::default-jet)) :accessor height        :initarg :height)
   (flue-labium-distance         :initform  (cadr (nth 6 mlys::default-jet)) :accessor flue-labium-distance      :initarg :flue-labium-distance)
   (labium-position  :initform  (cadr (nth 7 mlys::default-jet)) :accessor labium-position    :initarg :labium-position)
   (delta-scaler     :initform  (cadr (nth 8 mlys::default-jet)) :accessor delta-scaler    :initarg :delta-scaler)
   (ignore       :allocation :class :initform 1))
  (:documentation "A jet ")
  (:icon 579)
)
(setf (documentation (find-class 'Jet) 'type) (get-documentation 'objects 'Jet))
;(make-get-slot-in-out-names (find-class 'Jet))


(om::defclass! Open-Open-Tube (Simple-Object)
  ((tube-length      :initform  (cadr (nth 1 mlys::default-tube))  :accessor tube-length    :initarg :tube-length )
   (air-elasticity   :initform  (cadr (nth 2 mlys::default-tube))  :accessor air-elasticity :initarg :air-elasticity )
   (air-density      :initform  (cadr (nth 3 mlys::default-tube))  :accessor air-density    :initarg :air-density )
   (radius0          :initform  (cadr (nth 4 mlys::default-tube))  :accessor radius0        :initarg :radius0 )
   (radius1          :initform  (cadr (nth 5 mlys::default-tube))  :accessor radius1        :initarg :radius1  )
   (freq-loss        :initform  (cadr (nth 6 mlys::default-tube))  :accessor freq-loss      :initarg :freq-loss  )
   (const-loss       :initform  (cadr (nth 7 mlys::default-tube))  :accessor const-loss     :initarg :const-loss ))
  (:documentation "A tube open at both ends")
  (:icon 514))
(setf (documentation (find-class 'Open-Open-Tube) 'type) (get-documentation 'objects 'Open-Open-Tube))
(make-get-slot-in-out-names (find-class 'Open-Open-Tube))

(om::defclass! Mono-String (Simple-Object)
  ((string-length  :initform (cadr (nth 1 mlys::default-string)) :accessor string-length :initarg :string-length )
   (tension        :initform (cadr (nth 2 mlys::default-string)) :accessor tension :initarg :tension )
   (density        :initform (cadr (nth 3 mlys::default-string)) :accessor density :initarg :density )
   (radius         :initform (cadr (nth 4 mlys::default-string)) :accessor radius :initarg :radius )
   (young          :initform (cadr (nth 5 mlys::default-string)) :accessor young  :initarg :young  )
   (freq-loss      :initform (cadr (nth 6 mlys::default-string)) :accessor freq-loss  :initarg :freq-loss  )
   (const-loss     :initform (cadr (nth 7 mlys::default-string)) :accessor const-loss :initarg :const-loss ))
  (:documentation "A string or a rod that vibrates in one transverse direction")
  (:icon 500))
(setf (documentation (find-class 'Mono-String) 'type) (get-documentation 'objects 'Mono-String))
(make-get-slot-in-out-names (find-class 'Mono-String))

(om::defclass! Finite-Element (Simple-Object)
  ((mesh         :initform (cadr (nth 1 mlys::default-finite))   :type Mesh   :accessor mesh       :initarg :mesh)
   (density      :initform (cadr (nth 2 mlys::default-finite))   :type number :accessor density    :initarg :density)
   (thickness    :initform (cadr (nth 3 mlys::default-finite))   :type number :accessor thickness  :initarg :thickness)
   (young        :initform (cadr (nth 4 mlys::default-finite))   :type number :accessor young      :initarg :young)
   (poisson      :initform (cadr (nth 5 mlys::default-finite))   :type number :accessor poisson    :initarg :poisson)
   (freq-loss    :initform (cadr (nth 6 mlys::default-finite))   :type number :accessor freq-loss  :initarg :freq-loss)
   (const-loss   :initform (cadr (nth 7 mlys::default-finite))   :type number :accessor const-loss :initarg :const-loss)
   (blocked      :initform (cadr (nth 8 mlys::default-finite))   :type Mesh   :accessor blocked    :initarg :blocked))
  (:documentation "A object created from a mesh")
  (:icon 560))

(om::defclass! Free-Circ-Plate (Circ-Object)
  ((radius       :initform (cadr (nth 1 mlys::default-circplate))   :accessor radius     :initarg :radius)
   (thickness    :initform (cadr (nth 2 mlys::default-circplate))   :accessor thickness  :initarg :thickness)
   (density      :initform (cadr (nth 3 mlys::default-circplate))   :accessor density    :initarg :density)
   (young        :initform (cadr (nth 4 mlys::default-circplate))   :accessor young      :initarg :young)
   (poisson      :initform (cadr (nth 5 mlys::default-circplate))   :accessor poisson    :initarg :poisson)
   (freq-loss    :initform (cadr (nth 6 mlys::default-circplate))   :accessor freq-loss  :initarg :freq-loss)
   (const-loss   :initform (cadr (nth 7 mlys::default-circplate))   :accessor const-loss :initarg :const-loss))
  (:documentation "A circular plate unattached at its edges")
  (:icon 525))
(setf (documentation (find-class 'Free-Circ-Plate) 'type) (get-documentation 'objects 'Free-Circ-Plate))
(make-get-slot-in-out-names (find-class 'Free-Circ-Plate))

(om::defclass! Mono-Two-Mass (Simple-Object)
  ((small-mass   :initform (cadr (nth 0 mlys::default-mono-two-mass))  :accessor small-mass :initarg :small-mass)
   (large-mass   :initform (cadr (nth 1 mlys::default-mono-two-mass))  :accessor large-mass :initarg :large-mass)
   (stiffness1   :initform (cadr (nth 2 mlys::default-mono-two-mass))  :accessor stiffness1 :initarg :stiffness1)
   (freq-loss1   :initform (cadr (nth 3 mlys::default-mono-two-mass))  :accessor freq-loss1  :initarg :freq-loss1)
   (const-loss1  :initform (cadr (nth 4 mlys::default-mono-two-mass))  :accessor const-loss1 :initarg :const-loss1)
   (ignore       :allocation :class :initform 2))
  (:documentation "A simple two-mass model that can serve as a hammer, plectrum, finger, etc. Vibrates only in one direction")
  (:icon 573))
(setf (documentation (find-class 'Mono-Two-Mass) 'type) (get-documentation 'objects 'Mono-Two-Mass))
; somehow this perturbates everything for mono-two-mass
;(make-get-slot-in-out-names (find-class 'Mono-Two-Mass))

(om::defclass! Multiple-Points (Simple-Object)
  ((freqs       :initform  nil    :type list :accessor freqs :initarg :freqs)
   (bws         :initform  nil     :type list :accessor bws   :initarg :bws)
   (amps        :initform (list nil)      :type list :accessor amps  :initarg :amps))
  (:documentation "A Modalys Object without physical properties which can nevertheless be represented
in space with some points and can vibrate in one dimension")
(:icon 582))
(setf (documentation (find-class 'Multiple-Points) 'type) (get-documentation 'objects 'Multiple-Points))
;(make-get-slot-in-out-names (find-class 'Multiple-Points))

(om::defclass! Read-From-File (Simple-Object)
  ((file-name    :initform  nil     :type string :accessor file-name  :initarg :file-name)
   (ignore       :allocation :class :initform 2))
  (:documentation "Object created from modal data stored in a file")
  (:icon 549))
(setf (documentation (find-class 'Read-From-File) 'type) (get-documentation 'objects 'Read-From-File))
(make-get-slot-in-out-names (find-class 'Read-From-File))

(om::defclass! Rect-Free-Bar (Simple-Object)
  ((bar-length   :initform (cadr (nth 0 mlys::default-recfreeb))  :accessor bar-length :initarg :bar-length)
   (width        :initform (cadr (nth 1 mlys::default-recfreeb))  :accessor width      :initarg :width)
   (thickness    :initform (cadr (nth 2 mlys::default-recfreeb))  :accessor thickness  :initarg :thickness)
   (density      :initform (cadr (nth 3 mlys::default-recfreeb))  :accessor density    :initarg :density)
   (young0       :initform (cadr (nth 4 mlys::default-recfreeb))  :accessor young0     :initarg :young0)
   (young1       :initform (cadr (nth 5 mlys::default-recfreeb))  :accessor young1     :initarg :young1)
   (poisson      :initform (cadr (nth 6 mlys::default-recfreeb))  :accessor poisson    :initarg :poisson)
   (freq-loss    :initform (cadr (nth 7 mlys::default-recfreeb))  :accessor freq-loss  :initarg :freq-loss)
   (const-loss   :initform (cadr (nth 8 mlys::default-recfreeb))  :accessor const-loss :initarg :const-loss))
  (:documentation "A rectangular free bar")
  (:icon 562))
(setf (documentation (find-class 'Rect-Free-Bar) 'type) (get-documentation 'objects 'Rect-Free-Bar))
(make-get-slot-in-out-names (find-class 'Rect-Free-Bar))


(om::defclass! Rect-Object (Simple-Object)
  ())

(om::defclass! Rect-Membrane (Rect-Object)
  ((length-1     :initform (cadr (nth 1 mlys::default-recmem))  :accessor length-1   :initarg :length-1)
   (length-2     :initform (cadr (nth 2 mlys::default-recmem))  :accessor length-2   :initarg :length-2)
   (tension      :initform (cadr (nth 3 mlys::default-recmem))  :accessor tension    :initarg :tension)
   (surface-density  :initform (cadr (nth 4 mlys::default-recmem))  :accessor surface-density    :initarg :surface-density)
   (freq-loss    :initform (cadr (nth 5 mlys::default-recmem))  :accessor freq-loss  :initarg :freq-loss)
   (const-loss   :initform (cadr (nth 6 mlys::default-recmem))  :accessor const-loss :initarg :const-loss))
  (:documentation "A rectangular membrane of zero thickness, claped at its edges")
  (:icon 530))
(setf (documentation (find-class 'Rect-Membrane) 'type) (get-documentation 'objects 'Rect-Membrane))
(make-get-slot-in-out-names (find-class 'Rect-Membrane))

(om::defclass! Rect-Plate (Rect-Object)
  ((length-1     :initform (cadr (nth 1 mlys::default-recplate))  :accessor length-1   :initarg :length-1)
   (length-2     :initform (cadr (nth 2 mlys::default-recplate))  :accessor length-2   :initarg :length-2)
   (thickness    :initform (cadr (nth 3 mlys::default-recplate))  :accessor thickness  :initarg :thickness)
   (density      :initform (cadr (nth 4 mlys::default-recplate))  :accessor density    :initarg :density)
   (young        :initform (cadr (nth 5 mlys::default-recplate))  :accessor young      :initarg :young)
   (poisson      :initform (cadr (nth 6 mlys::default-recplate))  :accessor poisson    :initarg :poisson)
   (freq-loss    :initform (cadr (nth 7 mlys::default-recplate))  :accessor freq-loss  :initarg :freq-loss)
   (const-loss   :initform (cadr (nth 8 mlys::default-recplate))  :accessor const-loss :initarg :const-loss))
  (:documentation "A rectangular plate fixed at its edges")
  (:icon 510))
(setf (documentation (find-class 'Rect-Plate) 'type) (get-documentation 'objects 'Rect-Plate))
(make-get-slot-in-out-names (find-class 'Rect-Plate))

(om::defclass! Reson-Model (Simple-Object)
  ((maker        :initform (lambda (s file-path)
                             (let ((l (read-modres file-path)))
                               (make-object-basic 'single-point (length (first l)) (first l) (third l) (second l))))
                 :type function :allocation :class)
   (file-name    :initform nil     :type string :accessor file-name  :initarg :file-name)
   (ignore       :allocation :class :initform 2))
  (:documentation "Single point object whose mode frequencies, loss coefficients and amplitude vector are set to the values
of  model of resonance defined in a lisp parameter file.")
  (:icon 550))
(setf (documentation (find-class 'Reson-Model) 'type) (get-documentation 'objects 'Reson-Model))

(om::defclass! Single-Point (Simple-Object)
  ((freqs       :initform '(440)    :type list :accessor freqs :initarg :freqs)
   (bws         :initform '(10)     :type list :accessor bws   :initarg :bws)
   (amps        :initform '(1)      :type list :accessor amps  :initarg :amps))
  (:documentation "A Modalys Object without physical properties which can nevertheless be represented
in space and can vibrate in one dimension")
(:icon 551))
(setf (documentation (find-class 'Single-Point) 'type) (get-documentation 'objects 'Single-Point))
;(make-get-slot-in-out-names (find-class 'Single-Point))

(om::defclass! Violin-Bridge (Simple-Object)
  ((central-mass   :initform (cadr (nth 0 mlys::default-viobridge)) :accessor central-mass :initarg :central-mass)
   (feet-mass      :initform (cadr (nth 1 mlys::default-viobridge)) :accessor feet-mass :initarg :feet-mass)
   (e-distance     :initform (cadr (nth 2 mlys::default-viobridge)) :accessor e-distance :initarg :e-distance)
   (a-distance     :initform (cadr (nth 3 mlys::default-viobridge)) :accessor a-distance :initarg :a-distance)  
   (h-distance     :initform (cadr (nth 4 mlys::default-viobridge)) :accessor h-distance :initarg :h-distance)
   (b-distance     :initform (cadr (nth 5 mlys::default-viobridge)) :accessor b-distance :initarg :b-distance)
   (stiffness      :initform (cadr (nth 6 mlys::default-viobridge)) :accessor stiffness :initarg :stiffness)
   (freq-loss      :initform (cadr (nth 7 mlys::default-viobridge)) :accessor freq-loss  :initarg :freq-loss)
   (const-loss     :initform (cadr (nth 8 mlys::default-viobridge)) :accessor const-loss :initarg :const-loss)
   (ignore       :allocation :class :initform 2))
  (:documentation "A violin bridge")
  (:icon 546))
(setf (documentation (find-class 'Violin-Bridge) 'type) (get-documentation 'objects 'Violin-Bridge))
;(make-get-slot-in-out-names (find-class 'Violin-Bridge))

(om::defclass! Hybrid-Object (Modalys-Object)
  ((maker        :type function :initform (function modalys::make-object-basic)  :allocation :class :accessor maker)))

(om::defclass! Melt-Hybrid (Hybrid-Object)
  ((object1        :initform nil    :type Simple-Object :accessor object1 :initarg :object1)
   (object2        :initform nil    :type Simple-Object :accessor object2 :initarg :object2 )
   (controller     :initform nil    :type Controller    :accessor controller :initarg :controller ))
  (:documentation "Hybrid of two different objects. One can think of this hybrids the name implies, as one 
object melting into another. With it, the, one can make a string turning into a membrane, or have a tube tuned 
to 220 Hz turn into a tube tuned to 440 Hz")
  (:icon 520))
(setf (documentation (find-class 'Melt-Hybrid) 'type) (get-documentation 'objects 'Melt-Hybrid))

(om::defclass! Mix-Hybrid (Hybrid-Object)
  ((object1        :initform nil    :type Simple-Object :accessor object1 :initarg :object1)
   (object2        :initform nil    :type Simple-Object :accessor object2 :initarg :object2 )
   (controller     :initform nil    :type Controller    :accessor controller :initarg :controller ))
  (:documentation "A 'mis-hybrid can only be accessed using (make-hybrid-access ...) To do this, you must first make accesses
on the two objects from which the hybrid is derived. (make-hybrid-accss...) will keep track of where the access actually is")
  (:icon 531))
(setf (documentation (find-class 'Mix-Hybrid) 'type) (get-documentation 'objects 'Mix-Hybrid))


(om::defclass! Tri-Hybrid (Hybrid-Object)
  ((object1        :initform nil    :type Simple-Object :accessor object1 :initarg :object1)
   (object2        :initform nil    :type Simple-Object :accessor object2 :initarg :object2)
   (object3        :initform nil    :type Simple-Object :accessor object3 :initarg :object3)
   (controller     :initform nil    :type Controller    :accessor controller :initarg :controller ))
  (:documentation "Hybrid of three different objects."))
(setf (documentation (find-class 'Tri-Hybrid) 'type) (get-documentation 'objects 'Tri-Hybrid))



(om::defclass! Access (Modalys-Object)
  ((object       :initform nil :type Simple-Object :accessor object       :initarg    :object)
   (controller   :initform nil :type Controller :accessor controller   :initarg    :controller)
   (access-type  :initform nil :type symbol        :accessor access-type  :initarg    :access-type)
   (maker        :initform (function modalys::make-access) :type function :allocation :class))
  (:documentation "Access on an object")
  (:icon 502))

(defmethod om::get-slot-in-out-names ((self Access))
  (values 
   '("self" "object" "controller" "access-type")        ;;; noms des slots
   '(nil nil nil nil)                                ;;; valeur par defaut
   '("" "on which object" "where" "type of the access")        ;;; doc
   '(nil nil nil ((1 (("automatic" 'automatic)
                      ("trans0"    'trans0)
                      ("trans1"    'trans1)
                      ("normal"    'normal)
                      ("long"      'long)
                      ("decoded"   'decoded)
                      ("(x,y,z)"   'xyz)))))))

(om::defclass! Hybrid-Access (Modalys-Object)
  ((maker        :initform (function modalys::make-access-hybrid) :type function :allocation :class)
   (object       :initform nil    :type Hybrid-Object  :accessor object   :initarg :object)
   (access1      :initform nil    :type Access         :accessor access1  :initarg :access1)
   (access2      :initform nil    :type Access         :accessor access2  :initarg :access2))
  (:documentation "")
  (:icon 521))
(allow 'Hybrid-Access 'object (lambda (x) (or (null x)(typep x 'Hybrid-Object))))

(om::defclass! Tri-Hybrid-Access (Modalys-Object)
  ((maker        :initform (function modalys::make-access-tri-hybrid) :type function :allocation :class)
   (object       :initform nil    :type Hybrid-Object  :accessor object   :initarg :object)
   (access1      :initform nil    :type Access         :accessor access1  :initarg :access1)
   (access2      :initform nil    :type Access         :accessor access2  :initarg :access2)
   (access3      :initform nil    :type Access         :accessor access3  :initarg :access3))
  (:documentation "")
  (:icon 521))
(allow 'Tri-Hybrid-Access 'object (lambda (x) (or (null x)(typep x 'Hybrid-Object))))


(om::defclass! Connection (Modalys-Object)
  ((maker        :initform (function modalys::make-connection) :type function :allocation :class))
  (:documentation "A connection"))

(om::defclass! Adhere (Connection)
  ((access-1      :initform nil    :type Access   :accessor access-1   :initarg :access-1)
   (access-2      :initform nil    :type Access   :accessor access-2   :initarg :access-2)
   (weight           :initform nil    :type Controller :accessor weight          :initarg :weight))
  (:documentation "")
  (:icon 507))
(setf (documentation (find-class 'Adhere) 'type) (get-documentation 'connections 'Adhere))

(om::defclass! Bi-Fingerboard (Connection)
  ((finger-vert-acc    :initform nil    :type Access   :accessor finger-vert-acc   :initarg :finger-vert-acc)
   (finger-vert-pos    :initform 0      :type number   :accessor finger-vert-pos   :initarg :finger-vert-pos)
   (string-vert-acc    :initform nil    :type Access   :accessor string-vert-acc   :initarg :string-vert-acc)
   (string-vert-pos    :initform 0      :type number   :accessor string-vert-pos   :initarg :string-vert-pos)
   (finger-horiz-acc   :initform nil    :type Access   :accessor finger-horiz-acc   :initarg :finger-horiz-acc)
   (string-horiz-acc   :initform nil    :type Access   :accessor string-horiz-acc   :initarg :string-horiz-acc)
   (controller         :initform nil    :type Controller :accessor controller :initarg :controller)
   (weight           :initform nil    :type Controller :accessor weight          :initarg :weight))
  (:documentation "")
  (:icon 508))
(setf (documentation (find-class 'Bi-Fingerboard) 'type) (get-documentation 'connections 'Bi-Fingerboard))

(om::defclass! Bow (Connection)
  ((bow-vert-acc       :initform nil    :type Access   :accessor bow-vert-acc      :initarg :bow-vert-acc)
   (bow-horiz-acc      :initform nil    :type Access   :accessor bow-horiz-acc     :initarg :bow-horiz-acc)
   (bow-vert-pos       :initform 0      :type number   :accessor bow-vert-pos      :initarg :bow-vert-pos)
   (string-vert-acc    :initform nil    :type Access   :accessor string-vert-acc   :initarg :string-vert-acc)
   (string-horiz-acc   :initform nil    :type Access   :accessor string-horiz-acc  :initarg :string-horiz-acc)
   (string-vert-pos    :initform 0      :type number   :accessor string-vert-pos   :initarg :string-vert-pos)   
   (controller         :initform nil    :type Controller :accessor controller :initarg :controller)
   (weight           :initform nil    :type Controller :accessor weight          :initarg :weight))
  (:documentation "")
  (:icon 511))
(setf (documentation (find-class 'Bow) 'type) (get-documentation 'connections 'Bow))

(om::defclass! Felt (Connection)
  ((access-1      :initform nil :type Access   :accessor access-1   :initarg :access-1)
   (init-pos-1    :initform 0   :type number   :accessor init-pos-1 :initarg :init-pos-1)
   (access-2      :initform nil :type Access   :accessor access-2   :initarg :access-2)
   (init-pos-2    :initform 0   :type number   :accessor init-pos-2 :initarg :init-pos-2)
   (thickness     :initform nil :type Controller :accessor thickness  :initarg :thickness)
   (stiffness     :initform nil :type Controller :accessor stiffness :initarg :stiffness)
   (alpha         :initform nil :type Controller :accessor alpha     :initarg :alpha)
   (epsilon       :initform nil :type Controller :accessor epsilon   :initarg :epsilon)
   (tau           :initform nil :type Controller :accessor tau      :initarg :tau)
   (weight           :initform nil    :type Controller :accessor weight          :initarg :weight))
  (:documentation "Felt connection")
; without icon 
)
(setf (documentation (find-class 'Felt) 'type) (get-documentation 'connections 'Felt))
(allow 'Felt 'thickness (lambda (x) (or (numberp x)(typep x 'Controller))))
(allow 'Felt 'stiffness (lambda (x) (or (numberp x)(typep x 'Controller))))


(om::defclass! Force (Connection)
  ((access       :initform nil :accessor access     :initarg :access)
   (controller   :initform nil :accessor controller   :initarg :controller)
   (weight           :initform nil    :type Controller :accessor weight          :initarg :weight))
  (:documentation "")
  (:icon 526))
(setf (documentation (find-class 'Force) 'type) (get-documentation 'connections 'Force))
(allow 'Force 'controller (lambda (x) (or (null x)(numberp x)(typep x 'Controller))))

(om::defclass! Hole (Connection)
  ((access        :initform nil    :type Access   :accessor access   :initarg :access)
   (radius        :initform nil    :type Controller :accessor radius  :initarg :radius)
   (noise         :initform nil    :type Controller   :accessor noise :initarg :noise)
   (weight           :initform nil    :type Controller :accessor weight          :initarg :weight))
  (:documentation "")
  (:icon 516))
(setf (documentation (find-class 'Hole) 'type) (get-documentation 'connections 'Hole))


(om::defclass! Labium (Connection)
  ((access            :initform nil      :type Access       :accessor access   :initarg :access)
   (jet               :initform nil      :type Jet          :accessor jet   :initarg :jet)
   (noise             :initform nil      :type Controller   :accessor noise :initarg :noise)
   (vortex-factors    :initform nil      :type Controller   :accessor vortex-factors   :initarg :vortex-factors)
   (vortex-transition-time   :initform nil    :type Controller   :accessor vortex-transition-time   :initarg :vortex-transition-time)
   (weight           :initform nil       :type Controller   :accessor weight          :initarg :weight))
  (:documentation "")
)
(setf (documentation (find-class 'Labium) 'type) (get-documentation 'connections 'Labium))



(om::defclass! Mono-Fingerboard (Connection)
  ((finger-vert-acc    :initform nil    :type Access   :accessor finger-vert-acc   :initarg :finger-vert-acc)
   (finger-vert-pos    :initform 0      :type number   :accessor finger-vert-pos   :initarg :finger-vert-pos)
   (string-vert-acc    :initform nil    :type Access   :accessor string-vert-acc   :initarg :string-vert-acc)
   (string-vert-pos    :initform 0      :type number   :accessor string-vert-pos   :initarg :string-vert-pos)
   (controller         :initform nil    :type Controller :accessor controller :initarg :controller)
   (weight           :initform nil    :type Controller :accessor weight          :initarg :weight))
  (:documentation "")
  (:icon 572)
)
(setf (documentation (find-class 'Mono-Fingerboard) 'type) (get-documentation 'connections 'Mono-Fingerboard))



(om::defclass! Normalised-Valve (Connection)
  ((access-1        :initform nil  :type Access   :accessor access-1     :initarg :access-1)
   (init-pos-1      :initform 0    :type number   :accessor init-pos-1   :initarg :init-pos-1)
   (access-2        :initform nil  :type Access   :accessor access-2     :initarg :access-2)
   (init-pos-2      :initform 0    :type number   :accessor init-pos-2   :initarg :init-pos-2)
   (access-tube     :initform nil  :type Access   :accessor access-tube  :initarg :access-tube)
   (gamma           :initform nil  :type Controller :accessor gamma      :initarg :gamma)
   (air-density     :initform nil  :type Controller :accessor air-density   :initarg :air-density)
   (zeta            :initform nil  :type Controller :accessor zeta          :initarg :zeta)
   (front-angle     :initform nil  :type Controller :accessor front-angle    :initarg :front-angle)
   (aperture-length :initform nil  :type Controller :accessor aperture-length    :initarg :aperture-length)
   (weight          :initform nil  :type Controller :accessor weight         :initarg :weight))
  (:documentation "Normalised Valve Connection")
  (:icon 581))
(setf (documentation (find-class 'Normalised-Valve) 'type) (get-documentation 'connections 'Normalised-Valve))



(om::defclass! Position-Connection (Connection)
  ((maker        :initform (lambda (type access controller &optional weight) (modalys::make-connection 'position access controller weight))  :allocation :class)
   (access       :initform nil  :accessor access  :initarg :access)
   (controller   :initform nil :type Controller :accessor controller :initarg :controller)
   (weight           :initform nil    :type Controller :accessor weight          :initarg :weight))
  (:documentation "")
  (:icon 505))
(setf (documentation (find-class 'Position-Connection) 'type) (get-documentation 'connections 'Position))

(om::defclass! Pluck (Connection)
  ((access-1      :initform nil :type Access   :accessor access-1   :initarg :access-1)
   (init-pos-1    :initform 0   :type number   :accessor init-pos-1 :initarg :init-pos-1)
   (access-2      :initform nil :type Access   :accessor access-2   :initarg :access-2)
   (init-pos-2    :initform 0   :type number   :accessor init-pos-2 :initarg :init-pos-2)
   (controller    :initform nil :type Controller :accessor controller  :initarg :controller)
   (weight           :initform nil    :type Controller :accessor weight          :initarg :weight))
  (:documentation "")
  (:icon 504))
(setf  (documentation (find-class 'Pluck) 'type) (get-documentation 'connections 'Pluck))

(om::defclass! Reed (Connection)
  ((access-tube      :initform nil    :type Access     :accessor access-tube     :initarg :access-tube)
   (access-reed      :initform nil    :type Access     :accessor access-reed     :initarg :access-reed)
   (reed-separation  :initform 0      :type number     :accessor reed-separation :initarg :reed-separation)
   (breath           :initform nil    :type Controller :accessor breath          :initarg :breath)
   (reed-area        :initform nil    :type Controller :accessor reed-area       :initarg :reed-area)
   (weight           :initform nil    :type Controller :accessor weight          :initarg :weight))
  (:documentation "")
  (:icon 515))
(setf (documentation (find-class 'Reed) 'type) (get-documentation 'connections 'Reed))

(om::defclass! Strike (Connection)
  ((access-1      :initform nil  :type Access   :accessor access-1   :initarg :access-1)
   (init-pos-1    :initform 0    :type number   :accessor init-pos-1 :initarg :init-pos-1)
   (access-2      :initform nil  :type Access   :accessor access-2   :initarg :access-2)
   (init-pos-2    :initform 0    :type number   :accessor init-pos-2 :initarg :init-pos-2)
   (weight           :initform nil    :type Controller :accessor weight          :initarg :weight))
  (:icon 509))
(setf (documentation (find-class 'Strike) 'type) (get-documentation 'connections 'Strike))

(oa::om-with-redefinitions 
(om::defclass! Speed (Connection)
  ((access             :initform nil   :accessor access            :initarg :access)
   (controller         :initform nil  :type Controller :accessor controller :initarg :controller)
   (weight           :initform nil    :type Controller :accessor weight          :initarg :weight))
  (:documentation "")
  (:icon 527))
)
(setf (documentation (find-class 'mlys::Speed) 'type) (get-documentation 'connections 'Speed))

(om::defclass! Spring (Connection)
  ((access-1        :initform nil  :type Access :accessor access-1     :initarg :access-1)
   (access-2        :initform nil  :type Access :accessor access-2     :initarg :access-2)
   (stiffness       :initform nil  :type Controller :accessor stiffness     :initarg :stiffness)
   (freq-loss       :initform nil  :type Controller :accessor freq-loss     :initarg :freq-loss)
   (const-loss      :initform nil  :type Controller :accessor const-loss     :initarg :const-loss)
   (weight          :initform nil  :type Controller :accessor weight         :initarg :weight))
  (:documentation "Spring Connection")
  (:icon 574))
(setf (documentation (find-class 'mlys::Spring) 'type) (get-documentation 'connections 'Spring))

(om::defclass! Valve (Connection)
  ((access-1        :initform nil  :type Access   :accessor access-1     :initarg :access-1)
   (init-pos-1      :initform 0    :type number   :accessor init-pos-1   :initarg :init-pos-1)
   (access-2        :initform nil  :type Access   :accessor access-2     :initarg :access-2)
   (init-pos-2      :initform 0    :type number   :accessor init-pos-2   :initarg :init-pos-2)
   (access-tube     :initform nil  :type Access   :accessor access-tube  :initarg :access-tube)
   (air-pressure    :initform nil  :type Controller :accessor air-pressure   :initarg :air-pressure)
   (air-density     :initform nil  :type Controller :accessor air-density   :initarg :air-density)
   (under-area      :initform nil  :type Controller :accessor under-area    :initarg :under-area)
   (front-area      :initform nil  :type Controller :accessor front-area    :initarg :front-area)

   (front-angle     :initform nil  :type Controller :accessor front-angle    :initarg :front-angle)
   (aperture-length :initform nil  :type Controller :accessor aperture-length    :initarg :aperture-length)
   (weight          :initform nil  :type Controller :accessor weight         :initarg :weight))
  (:documentation "Valve Connection")
  (:icon 580))
(setf (documentation (find-class 'Valve) 'type) (get-documentation 'connections 'Valve))


(om::defclass! Point-Output (Modalys-Object)
  ((access       :initform nil   :accessor access   :initarg :access)
   (channel-num  :initform 0  :type integer :accessor channel-num :initarg :channel-num)
   (controller   :initform nil  :accessor controller :initarg :controller)
   (modes-selection   :initform nil  :type list       :accessor modes-selection :initarg :modes-selection)
   (maker        :initform (function make-point-output) :allocation :class))
   (:documentation "Audio output at an access")
   (:icon 512))
; unfortunately, this allow does not work in OM 5.0.1 k ... to check
;(allow 'mlys::Point-Output 'mlys::access (lambda (x) (or (null x) (typep x 'mlys::Controller)(typep x 'mlys::Access)(typep x 'mlys::Hybrid-Access))))

(om::defclass! Point-Input (Modalys-Object)
  ((channel-num  :initform 0    :type integer  :accessor channel-num :initarg :channel-num)
   (scaler       :initform nil    :type Controller :accessor scaler :initarg :scaler)
   (maker        :initform (function make-point-input) :allocation :class))
   (:documentation "Audio input")
   (:icon 566))

(om::defclass! Access-Speed (Controller)
  ((access             :initform nil  :type Access  :accessor access           :initarg :access))
  (:icon 569))

(setf (documentation (find-class 'Access-Speed) 'type) (get-documentation 'controllers 'Access-Speed))

(om::defclass! Access-Position (Controller)
  ((access             :initform nil  :type Access  :accessor access           :initarg :access))
  (:icon 570))

(setf (documentation (find-class 'Access-Position) 'type) (get-documentation 'controllers 'Access-Position))

(om::defclass! Access-Force (Controller)
  ((access             :initform nil  :type Access  :accessor access           :initarg :access))
  (:icon 571))

(setf (documentation (find-class 'Access-Force) 'type) (get-documentation 'controllers 'Access-Force))

;
;
;    METHODS
;

(om::defmethod! om::om* ((first-arg Controller) (second-arg Controller))
  (make-instance 'arithmetic 
    :dimension (dimension first-arg)
    :operation '*
    :controllers (list first-arg second-arg)))

(om::defmethod! om::om+ ((first-arg Controller) (second-arg Controller))
  (make-instance 'arithmetic 
    :dimension (dimension first-arg)
    :operation '+
    :controllers (list first-arg second-arg)))

(om::defmethod! om::om- ((first-arg Controller) (second-arg Controller))
  (make-instance 'arithmetic 
    :dimension (dimension first-arg)
    :operation '-
    :controllers (list first-arg second-arg)))

(om::defmethod! om::om/ ((first-arg Controller) (second-arg Controller))
  (make-instance 'arithmetic 
    :dimension (dimension first-arg)
    :operation '/
    :controllers (list first-arg second-arg)))

(om::defmethod! mview ((o mlys::Mesh) &rest arguments) 
  :icon 565
  (mlys::view-internal (cons 'mesh (cons o arguments))))

(om::defmethod! mview ((o mlys::Finite-Element) &rest arguments) 
  :icon 565
  (mlys::view-internal (concatenate 'list (list 'mode o) arguments)))

(if (>= om::*om-version* 5.0)
  (om::defmethod! duplicate ((a symbol) (m Mesh) &rest arguments) 
    :icon 565
    :menuins '((0 (("translation" 'modalys::translation) ("rotation" 'modalys::rotation)
                   ("helix" 'modalys::helix)("homothety" 'modalys::homothety)("reflection" 'modalys::reflection))))
    (progn
      (duplicate-internal (cons a (cons m arguments)))
      m))
  (om::defmethod! duplicate ((a symbol) (m Mesh) &rest arguments) 
    :icon 565
    :menuins ((0 (("translation" 'modalys::translation) ("rotation" 'modalys::rotation)
                   ("helix" 'modalys::helix)("homothety" 'modalys::homothety)("reflection" 'modalys::reflection))))
    (progn
      (duplicate-internal (cons a (cons m arguments)))
      m)))

(om::defmethod! transform ((a symbol) (m Mesh) &rest arguments) :icon 565
  (transform-internal (cons a (cons m arguments))))

(om::defmethod! const ((x list)) :icon 513
  (make-instance 'CONSTANT :dimension (length x) :value x))
(om::defmethod! const ((x number)) :icon 513
  (make-instance 'CONSTANT :dimension 1 :value x))

(om::defmethod! set-breakpoint ((dynamic Controller) &rest arguments) :icon 565
  (let ((p (slot-value dynamic 'private-ptr))) 
    (if p 
      (mlys::set-breakpoint-internal p arguments))))

(defun has-nil (l)
  (if (not l)
    nil
    (if (or (not (car l)) (null-ptr-p (car l)))
      t
      (has-nil (cdr l)))))

(defun has-at-least-a-null-arg (l)
  (if (not l)
    nil
    (if (and  (car l) (null-ptr-p (car l)))
      t
      (has-at-least-a-null-arg (cdr l)))))

(defun get-pointer (x)
  (if (is-modalys-object x)
    (private-ptr x)
    x))

(defun value-conversion-single (val type)
  (cond
   ((is-modalys-object val)  (private-ptr val))
   ((and (numberp val) (eq type 'mlys::controller))
                             (private-ptr (const val)))
   (t                        val)))

(defun value-conversion-general (val type)
  (if (listp val)
    (if (eq type 'mlys::controller)
      (private-ptr (const val))
      (loop for v in val
            collect
            (value-conversion-single v type)))
    (value-conversion-single val type)))

(defun object-values-to-list (object)
  (let
    ((name-getter (function clos:slot-definition-name))
     (type-getter (function clos:slot-definition-type)))
    (let*
        ((c (class-of object))
         (sl 
          #+mcl   (class-instance-slots c)
          #+lispworks (om-api:class-instance-slots c)
          ))
      (loop
       for slot in (mapcar name-getter sl)
       for type in (mapcar type-getter sl)
       collect
       (let ((val (slot-value object slot)))
         (value-conversion-general val type))))))

(defun loading ()
  om::*loaading-stack*)

;; lors de la creation des classes modalys, les arguments se font controler
;; via le predicat 'type-predicate de la classe effective-slot-definition associe au
;; slot en question. Pour ces slots, nous avons besoin d'un peu de souplesse
;; notamment, un slot attendant un controlleur peut recevoir un nombre, car c'est trivial
;; de le convertir. Cet ajustement est fait ici

(defun adjust-slots-predicate (c)
  (loop for x in (class-instance-slots c)
       do
       (if (eq (slot-value x 'ccl::type) 'ccl::vector)
         (setf (slot-value x 'ccl::type-predicate) (lambda (x) (or (null x)(vectorp x)(listp x)))))
       (if (eq (slot-value x 'ccl::type) 'Simple-Object)
         (setf (slot-value x 'ccl::type-predicate) (lambda (x) (or (null x)(typep x 'Simple-Object)))))
       (if (eq (slot-value x 'ccl::type) 'Mesh)
         (setf (slot-value x 'ccl::type-predicate) (lambda (x) (or (null x)(typep x 'Mesh)))))
       (if (eq (slot-value x 'ccl::type) 'Controller)
         (setf (slot-value x 'ccl::type-predicate) (lambda (x) (or (null x)(numberp x) (listp x) (typep x 'Controller)))))
       (if (eq (slot-value x 'ccl::type) 'Access)
         (setf (slot-value x 'ccl::type-predicate) (lambda (x) (or (null x) (typep x 'Access) (typep x 'Hybrid-Access) (typep x 'Tri-Hybrid-Access)))))
       (if (eq (slot-value x 'ccl::type) 'ccl::integer)
         (setf (slot-value x 'ccl::type-predicate) (lambda (x) (or (null x) (integerp x )))))
       (if (eq (slot-value x 'ccl::type) 'ccl::number)
         (setf (slot-value x 'ccl::type-predicate) (lambda (x) (or (null x) (numberp x )))))
       (if (eq (slot-value x 'ccl::name) 'file-name)
         (setf (slot-value x 'ccl::type-predicate) (lambda (x) (or (null x) (stringp x) (pathnamep x))))))
  (loop for sub-class in (slot-value c 'ccl::direct-subclasses)
        do (adjust-slots-predicate sub-class)))

#+mcl (if (>= om::*om-version* 5.01)
#+mcl   (adjust-slots-predicate (find-class 'Modalys-Object)))

(defmethod setting-inner-ptr ((self Modalys-Object))
  (let ((l (nthcdr (slot-value self 'ignore) (object-values-to-list self))))
    (if (and l (not (has-nil l)))
           (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) l)))))

(defmethod setting-inner-ptr ((self Mesh))
  (let ((l (nthcdr (slot-value self 'ignore) (object-values-to-list self))))
    (if (and l (not (has-nil l)))
      (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) 
                                                  (cons (which self)  l))))))

(defmethod setting-inner-ptr ((self Point-Output))
  (let ((l (nthcdr (slot-value self 'ignore) (object-values-to-list self))))
    (if (first l)
      (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) l)))))

(defmethod setting-inner-ptr ((self Point-Input))
  (let ((l (nthcdr (slot-value self 'ignore) (object-values-to-list self))))
    (if (first l)
      (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) l)))))

(defmethod setting-inner-ptr ((self Controller))
  (let ((l (nthcdr (slot-value self 'ignore) (object-values-to-list self))))
(print (format nil "HERE controller class ~A ~A" (class-of self) l))))


(defmethod setting-inner-ptr ((self Controller))
  (let ((l (nthcdr (slot-value self 'ignore) (object-values-to-list self))))
    (if (and l (not (has-nil l))  (not (eql 'mlys::controller (class-name (class-of self)))))
      (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) (cons (class-name (class-of self)) l))))))

(defmethod setting-inner-ptr ((self Noise))
  (let ((l 
         (nthcdr (slot-value self 'ignore) (object-values-to-list self))))
    (if (and l (not (has-nil l)))
      (let ((ll (list (first l) (second l) 
                      (list (third l) (fourth l) (fifth l) (sixth l) (seventh l)) (nth 7 l))))
        (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker)
                                                    (cons (class-name (class-of self)) ll)))))))

(defmethod setting-inner-ptr ((self Connection))
  (let ((l (nthcdr (slot-value self 'ignore) (object-values-to-list self))))
    (if (and l (not (has-nil (butlast l 1))))
      (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) (cons (class-name (class-of self)) l))))))


(defmethod setting-inner-ptr ((self Valve))
  (let ((l (nthcdr (slot-value self 'ignore) (object-values-to-list self))))
    (if (first l)
        (progn
          (if (not (third l))
              (setf l (cons (first l) (cons (second l) (nthcdr 4 l)))))
          (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) (cons (class-name (class-of self)) l)))))))

(defmethod setting-inner-ptr ((self Normalised-Valve))
  (let ((l (nthcdr (slot-value self 'ignore) (object-values-to-list self))))
    (if (first l)
        (progn
          (if (not (third l))
              (setf l (cons (first l) (cons (second l) (nthcdr 4 l)))))
          (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) (cons (class-name (class-of self)) l)))))))


(defmethod setting-inner-ptr ((self Simple-Object))
  (if (not (eq (class-name (class-of self)) 'Simple-Object))
    (let ((l (nthcdr (+(slot-value self 'ignore) 3) (object-values-to-list self))))
      (if (not (has-nil l))
        ;(print l)))))
        (progn
          (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) 
                                                      (cons (class-name (class-of self)) 
                                                            l)))
          (if (compiled-access self)
            (set-compiled-access (private-ptr self) (compiled-access self) ))
          (if (tuning-param self)
            (set-pitch (private-ptr self) (tuning-param self) (tuning-freq self))))))))

(defmethod setting-inner-ptr ((self Wave-Guide))
  (if (not (eq (class-name (class-of self)) 'Wave-Guide))
    (let ((l (nthcdr (+(slot-value self 'ignore) 3) (object-values-to-list self))))
      (if (not (has-nil l))
       ; (print (cons (class-name (class-of self)) l))))))
        (progn
          (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) 
                                                      (cons (class-name (class-of self)) 
                                                            l)))
          (if (compiled-access self)
            (set-compiled-access (private-ptr self) (compiled-access self) ))
          (if (tuning-param self)
            (set-pitch (private-ptr self) (tuning-param self) (tuning-freq self))))))))



(defmethod setting-inner-ptr ((self Finite-Element))
    (let ((l (nthcdr (+(slot-value self 'ignore) 3) (object-values-to-list self))))
; do something only if we have a mesh in the appropriate slot
      (if (second l)
        (let 
          ((nm
            (if (pointerp (first l))
                (get-info 'value (first l) 0)
                (first l))))
          (setf (first l) nm)
          (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) 
                                                      (cons (class-name (class-of self)) 
                                                            l)))
          (compute-modes (private-ptr self))))))





(defmethod setting-inner-ptr ((self Dynamic))
  (let ((l (nthcdr (slot-value self 'ignore) (object-values-to-list self))))
    (if (and l (first l) (second l) (third l))
      (progn
        (if (not (fourth l))
          (setf (fourth l) ""))
        (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) (cons (class-name (class-of self)) l)))))))

(defmethod setting-inner-ptr ((self Envelope))
  (let ((l (nthcdr (slot-value self 'ignore) (object-values-to-list self))))
    (if (and l (not (has-nil l)))
      (let ((b (second l)))
        (if (typep b 'OM::BPF)
          (setf (second l) (om::mat-trans (list (om::x-points b) (om::y-points b)))))
        (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) (cons (class-name (class-of self)) l)))))))

(defmethod setting-inner-ptr ((self Hybrid-Object))
  (let ((l (nthcdr (slot-value self 'ignore) (object-values-to-list self))))
    (if (and l (not (has-nil l)))
      (setf (slot-value self 'private-ptr) (apply (slot-value self 'maker) 
                                              (cons (class-name (class-of self)) l))))))

(defmethod setting-inner-ptr ((self Access))
  (if (and (object self) (controller self) (access-type self))
    (let* (
           (mapping
            (list
             (cons 'mono-string         'trans0)
             (cons 'bi-string           'trans0)
             (cons 'bi-two-mass         'trans0)
             (cons 'cello-bridge        'base)
             (cons 'closed-closed-tube  'long)
             (cons 'closed-open-tube    'long)
             (cons 'jet                 'foot)
             (cons 'mono-two-mass       'trans0)
             (cons 'violin-bridge       'base)))
           (access-type-final
            (if (and (access-type self) (eq (access-type self) 'automatic))
                (let ((at
                       (let ((cn (class-name (class-of (object self)))))
                         (cdr (assoc (if (eq cn 'clone)
                                         (class-name (class-of (original (object self))))
                                       cn)
                                     mapping)))))
                  (if at at 'normal))
              (access-type self)))
           )
      (setf (slot-value self 'private-ptr)
            (apply (maker self) (list (private-ptr (object self)) (controller self) access-type-final))))))


(defmethod setting-inner-ptr ((self Plot))
  (setf (slot-value self 'private-ptr) (funcall (slot-value self 'maker))))







(defclass ModalysEvalOnceBox (om::OMBoxEditCall) ())
(defmethod om::get-type-of-ed-box ((self Simple-Object))  'ModalysEvalOnceBox)
(defmethod om::get-type-of-ed-box ((self Hybrid-Object))  'ModalysEvalOnceBox)

(defmethod initialize-instance :before ((self ModalysEvalOnceBox) &rest args)
 (setf (om::allow-lock self) "&"))


(defmethod initialize-instance :after ((self Modalys-Object) &rest args)
  (declare (ignore args))
  (if (not (loading))
    (setting-inner-ptr self))
	)


(in-package OM)

(defmethod om::omNG-save ((self mlys::Hybrid-Access) &optional (values? nil))
   "Cons a Lisp expression that returns a copy of self when it is evaluated."
   `(let ((mhy ,(om::omng-save (mlys::object self))))
      (make-instance ',(type-of self) 
        :object mhy
        :access1 (make-instance 'mlys::access 
                   :object (if mhy (mlys::object1 mhy))
                   :controller ,(om::omng-save (if  (mlys::access1 self) (mlys::controller (mlys::access1 self))))
                   :access-type ,(om::omng-save (if (mlys::access1 self) (mlys::access-type (mlys::access1 self))))
                   )
        :access2 (make-instance 'mlys::access 
                   :object (if mhy (mlys::object2 mhy))
                   :controller ,(om::omng-save (if (mlys::access2 self) (mlys::controller (mlys::access2 self))))
                   :access-type ,(om::omng-save (if (mlys::access2 self) (mlys::access-type (mlys::access2 self))))
                   ))))

(defmethod om::omNG-save ((self mlys::Tri-Hybrid-Access) &optional (values? nil))
   "Cons a Lisp expression that returns a copy of self when it is evaluated."
   `(let ((mhy ,(om::omng-save (mlys::object self))))
      (make-instance ',(type-of self) 
        :object mhy
        :access1 (make-instance 'mlys::access 
                   :object (if mhy (mlys::object1 mhy))
                   :controller ,(om::omng-save (if  (mlys::access1 self) (mlys::controller (mlys::access1 self))))
                   :access-type ,(om::omng-save (if (mlys::access1 self) (mlys::access-type (mlys::access1 self))))
                   )
        :access2 (make-instance 'mlys::access 
                   :object (if mhy (mlys::object2 mhy))
                   :controller ,(om::omng-save (if (mlys::access2 self) (mlys::controller (mlys::access2 self))))
                   :access-type ,(om::omng-save (if (mlys::access2 self) (mlys::access-type (mlys::access2 self))))
                   )
        :access3 (make-instance 'mlys::access 
                   :object (if mhy (mlys::object3 mhy))
                   :controller ,(om::omng-save (if (mlys::access3 self) (mlys::controller (mlys::access3 self))))
                   :access-type ,(om::omng-save (if (mlys::access3 self) (mlys::access-type (mlys::access3 self))))
                   ))))



(defmethod omNG-save ((self mlys::Point-Output) &optional (values? nil))
   "Cons a Lisp expression that returns a copy of self when it is evaluated."
   `(make-instance ',(type-of self) 
        :access ,(omng-save (mlys::access self))
        :channel-num nil
        :controller ,(omng-save (mlys::controller self))
        :modes-selection nil
        ))

(defmethod omNG-copy ((self mlys::Speed))
   `(make-instance ',(class-name (class-of self))))

(defmethod omNG-copy ((self mlys::Random))
   `(make-instance ',(class-name (class-of self))))

(defmethod omNG-copy ((self mlys::Finite-Element))
   `(make-instance ',(class-name (class-of self))))

(defmethod omNG-copy ((self mlys::Mesh-Restrict-Quadrilateral))
   `(make-instance ',(class-name (class-of self))
      :original ,(mlys::original self)
      :quadrilateral ',(mlys::quadrilateral self)))

(defmethod omNG-copy ((self mlys::Access))
   `(make-instance ',(class-name (class-of self))
      :object nil))

(defmethod omNG-copy ((self mlys::Point-Output))
   `(make-instance ',(class-name (class-of self))
      :access nil))



(in-package "MODALYS")

(om::defclass! Component ()())

;;;
;;;   Key
;;;

(om::defclass! key (Component) ((my-access) (my-radius-ctlr) (my-hole-cn) 
                       (radius :initarg :radius :initform nil :accessor radius)
                       (tube :initarg :tube :accessor tube :initform nil)
                       (key-semitones :initarg :key-semitones :accessor key-semitones :initform 12))
  (:icon 532))

;;;
;;; procedure for making a "key" (initially closed) on an acoustic tube.
;;; user specifies the number of semitones above the tube's pitch that the
;;; key should correspond to.
;;;

(defmethod initialize-instance :after ((o key) &rest args)
  (declare (ignore args))
  (if (not (loading))
    (if (slot-value o 'tube)
      (progn 
        (if (<= (slot-value o 'key-semitones) 0)
          (error 'key-component "key-semitones should be greater than zero"))
        (let ((loc (exp (* (log 2) (/ (slot-value o 'key-semitones) -12)))))
          (setf (slot-value o 'my-access) (make-instance 'access :object (slot-value o 'tube) :controller loc :access-type 'long))
          (setf (slot-value o 'my-radius-ctlr) (make-instance 'dynamic :dimension 1 :update_interval 0.001 :value (list 0)))
          (setf (slot-value o 'my-hole-cn) (make-instance 'hole :access (slot-value o 'my-access)
                                                           :radius (slot-value o 'my-radius-ctlr) :noise 0)))))))

;;;
;;; "open" messages
;;;


(defmethod key/open ((self key))
  (set-breakpoint (slot-value self 'my-radius-ctlr) (list .02 (slot-value self 'radius))))

(defmethod key/open-in-time ((self key) time)
  (set-breakpoint (slot-value self 'my-radius-ctlr) (list time (slot-value self 'radius))))

;;;
;;; "close" messages
;;;

(defmethod key/close ((self key))
  (set-breakpoint (slot-value self 'my-radius-ctlr) (list .02 0)))

(defmethod key/close-in-time ((self key) time)
  (set-breakpoint (slot-value self 'my-radius-ctlr) (list time 0)))


;;;
;;;   Hammer
;;;

(in-package "MODALYS")

(om::defclass! hammer (Component) 
  ((striker                   :accessor striker :initarg :striker :initform nil)
   (strikee                   :accessor strikee :initarg :strikee :initform nil)
   (period                    :accessor period  :initarg :period  :initform nil)
   (my-delay)
   (head                      :accessor head)
   (handle                    :accessor handle)
   (strike-cn                 :accessor strike-cn)
   (force :initform 0.0       :accessor force)
   (state :initform 'waiting  :accessor state))
  (:icon 533))

(defmethod initialize-instance :after ((o hammer) &rest args)
  (declare (ignore args))
  (if (not (loading))
    (if (striker o)
      (progn
        (setf (slot-value o 'striker) (private-ptr (striker o)))
        (setf (slot-value o 'strikee) (private-ptr (strikee o)))
        (setf (slot-value o 'head) (make-access (striker o) (private-ptr (const 0)) 'trans0))
        (setf (slot-value o 'handle) (make-access (striker o) (private-ptr (const 1)) 'trans0))
        (freeze-object (striker o))
        (let ((scn-thunk (lambda (inputs output)
                           (declare (ignore inputs))
                           (case (state o)
                             ((accelerating)
                              (if (> 0 (get-info 'position (head o)))
                                (progn
                                  (setf (force o) 0.0)
                                  (setf (state o) 'touching))))
                             ((touching)
                              (if (< 0 (get-info 'position (head o)))
                                (setf (state o) 'bouncing)))
                             ((bouncing)
                              (progn
                                (print "want to destroy & freeze")
                                (destroy-connection (strike-cn o))
                                ;(freeze-object (striker o))
                                (setf (state o) 'waiting))))
                           (vset output 0 (force o)))))
          (make-connection 'force (handle o)
                           (make-controller 'foreign-call 1 (period o) scn-thunk)))))))

(defmethod strike-hammer ((o hammer) delay velocity)
  (if (eq (state o) 'waiting)
      (let ((position (* (/ velocity 2) delay)))
        (unfreeze-object (striker o))
        (set-access-displacement (handle o) position)
        (setf (state o) 'accelerating)
        (setf (force o) (- (/ (* velocity (get-info 'mass (striker o))) delay)))
        (setf (strike-cn o) (make-connection 'strike (head o) position (strikee o) 0))
        )
        ; else 
        (error 'hammer-component
               "hammer is in ~a state, cannot strike" (state o))))

(defmethod state? ((o hammer)) (state o))

(if (>= om::*om-version* 5.0)
  (om::defmethod! mlys::synthesize ((sound-file-name-determination-mode symbol) 
                              (sound-file-or-directory-in t)
                              &rest arguments)
    :initvals '('modalys::system nil)
    :menuins '((0 (("system determined" 'modalys::system) ("user determined" 'modalys::user))))
    :icon 561

    (let* ((sound-file-or-directory (if (and sound-file-or-directory-in (stringp sound-file-or-directory-in)) 
                                        (mlys::pathname-from-posix sound-file-or-directory-in)
                                      sound-file-or-directory-in))
           (file-name (if (eql sound-file-name-determination-mode 'system) 
                          (get-temporary-file "aiff")
                        (if (om::directoryp sound-file-or-directory)
                            (merge-pathnames 
                             (make-pathname :name (format nil "modalys-~D" (get-internal-real-time)) :type "aiff")
                             sound-file-or-directory)
                          (let ((tmp sound-file-or-directory))
                            (if (eq (pathname-type tmp) :unspecific)
                                (make-pathname :directory (directory-namestring tmp) 
                                               :name (file-namestring tmp) :type "aiff")
                              tmp)) 
                          )
                        )))
      (mlys::save-sound file-name :file-format 'mlys::aiff)
      (if (stringp file-name)
          (mlys::pathname-from-posix file-name)
        file-name)))
  (om::defmethod! synthesize ((sound-file-name-determination-mode symbol) 
                              (sound-file-or-directory t)
                              &rest arguments)
    :initvals '('modalys::system nil)
    :menuins ((0 (("system determined" 'modalys::system) ("user determined" 'modalys::user))))
    :icon 561
    
    (let ((file-name (if (eql sound-file-name-determination-mode 'system) 
                       (get-temporary-file "aiff")
                       (if (om::directoryp sound-file-or-directory)
                         (progn
                           (make-pathname :directory (directory-namestring sound-file-or-directory)
                                          :name (format nil "modalys-~D" (get-tick-count)) :type "aiff"))
                         (let ((tmp sound-file-or-directory))
                           (if (eq (pathname-type tmp) :unspecific)
                             (make-pathname :directory (directory-namestring tmp) 
                                            :name (file-namestring tmp) :type "aiff")
                             tmp)) 
                         ))))
      (mlys::save-sound file-name :file-format 'mlys::aiff)
      (if (stringp file-name)
        (mlys::pathname-from-posix file-name)
        file-name))))

(defvar *simple-object-fond* nil)
(defvar *io-fond* nil)
(defvar *connection-fond* nil)
(defvar *controller-fond* nil)
(defvar *hybrid-fond* nil)
(defvar *mesh-fond* nil)
(defvar *component-fond* nil)

(if (>= om::*om-version* 5.0)
  (progn
    (setq mlys::res-dir (make-pathname :directory (append (pathname-directory ol) (list "resources" "pict"))))
    (setq mlys::*simple-object-fond* 
          (om::om-load-and-store-picture "object" 'om::full mlys::res-dir))
    (defmethod om::get-fonde-pict ((self mlys::Simple-Object)) mlys::*simple-object-fond*)
    
    (setq mlys::*io-fond* 
          (om::om-load-and-store-picture "io" 'om::full mlys::res-dir))
    (defmethod om::get-fonde-pict ((self mlys::Access)) mlys::*io-fond*)
    (defmethod om::get-fonde-pict ((self mlys::Point-Output)) mlys::*io-fond*)
    (defmethod om::get-fonde-pict ((self mlys::Point-Input)) mlys::*io-fond*)
    (defmethod om::get-fonde-pict ((self mlys::Hybrid-Access)) mlys::*io-fond*)
    (defmethod om::get-fonde-pict ((self mlys::Tri-Hybrid-Access)) mlys::*io-fond*)

    (setq mlys::*connection-fond* 
          (om::om-load-and-store-picture "connection" 'om::full mlys::res-dir))
    (defmethod om::get-fonde-pict ((self mlys::Connection)) mlys::*connection-fond*)
    
    (setq mlys::*controller-fond* 
          (om::om-load-and-store-picture "controller" 'om::full mlys::res-dir))
    (defmethod om::get-fonde-pict ((self mlys::Controller)) mlys::*controller-fond*)
    
    (setq mlys::*hybrid-fond* 
          (om::om-load-and-store-picture "hybrid" 'om::full mlys::res-dir))
    (defmethod om::get-fonde-pict ((self mlys::Hybrid-Object)) mlys::*simple-object-fond*)
    
    (setq mlys::*mesh-fond* 
          (om::om-load-and-store-picture "mesh" 'om::full mlys::res-dir))
    (defmethod om::get-fonde-pict ((self mlys::Mesh)) mlys::*mesh-fond*)
    (setq mlys::*component-fond* 
          (om::om-load-and-store-picture "component" 'om::full mlys::res-dir))
    (defmethod om::get-fonde-pict ((self mlys::Component)) mlys::*component-fond*)
    ))

